# -*- coding: utf-8 -*-
# Copyright 2010-2011 OpenStack Foundation
# Copyright 2014 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime
import gettext
import http.client as http
import os
import socket
from unittest import mock

import eventlet.patcher
import fixtures
from oslo_concurrency import processutils
from oslo_serialization import jsonutils
import routes
import webob

from glance.api.v2 import router as router_v2
from glance.common import exception
from glance.common import utils
from glance.common import wsgi
from glance import i18n
from glance.image_cache import prefetcher
from glance.tests import utils as test_utils


class RequestTest(test_utils.BaseTestCase):

    def _set_expected_languages(self, all_locales=None, avail_locales=None):
        if all_locales is None:
            all_locales = []

        # Override gettext.find to return other than None for some languages.
        def fake_gettext_find(lang_id, *args, **kwargs):
            found_ret = '/glance/%s/LC_MESSAGES/glance.mo' % lang_id
            if avail_locales is None:
                # All locales are available.
                return found_ret
            languages = kwargs['languages']
            if languages[0] in avail_locales:
                return found_ret
            return None

        self.mock_object(gettext, 'find', fake_gettext_find)

    def test_content_range(self):
        request = wsgi.Request.blank('/tests/123')
        request.headers["Content-Range"] = 'bytes 10-99/*'
        range_ = request.get_range_from_request(120)
        self.assertEqual(10, range_.start)
        self.assertEqual(100, range_.stop)  # non-inclusive
        self.assertIsNone(range_.length)

    def test_content_range_invalid(self):
        request = wsgi.Request.blank('/tests/123')
        request.headers["Content-Range"] = 'bytes=0-99'
        self.assertRaises(webob.exc.HTTPRequestRangeNotSatisfiable,
                          request.get_range_from_request, 120)

    def test_range(self):
        request = wsgi.Request.blank('/tests/123')
        request.headers["Range"] = 'bytes=10-99'
        range_ = request.get_range_from_request(120)
        self.assertEqual(10, range_.start)
        self.assertEqual(100, range_.end)  # non-inclusive

    def test_range_invalid(self):
        request = wsgi.Request.blank('/tests/123')
        request.headers["Range"] = 'bytes=150-'
        self.assertRaises(webob.exc.HTTPRequestRangeNotSatisfiable,
                          request.get_range_from_request, 120)

    def test_content_type_missing(self):
        request = wsgi.Request.blank('/tests/123')
        self.assertRaises(exception.InvalidContentType,
                          request.get_content_type, ('application/xml',))

    def test_content_type_unsupported(self):
        request = wsgi.Request.blank('/tests/123')
        request.headers["Content-Type"] = "text/html"
        self.assertRaises(exception.InvalidContentType,
                          request.get_content_type, ('application/xml',))

    def test_content_type_with_charset(self):
        request = wsgi.Request.blank('/tests/123')
        request.headers["Content-Type"] = "application/json; charset=UTF-8"
        result = request.get_content_type(('application/json',))
        self.assertEqual("application/json", result)

    def test_params(self):
        expected = webob.multidict.NestedMultiDict({
            'limit': '20', 'name': 'Привет', 'sort_key': 'name',
            'sort_dir': 'asc'})

        request = wsgi.Request.blank("/?limit=20&name=%D0%9F%D1%80%D0%B8"
                                     "%D0%B2%D0%B5%D1%82&sort_key=name"
                                     "&sort_dir=asc")
        actual = request.params
        self.assertEqual(expected, actual)

    def test_content_type_from_accept_xml(self):
        request = wsgi.Request.blank('/tests/123')
        request.headers["Accept"] = "application/xml"
        result = request.best_match_content_type()
        self.assertEqual("application/json", result)

    def test_content_type_from_accept_json(self):
        request = wsgi.Request.blank('/tests/123')
        request.headers["Accept"] = "application/json"
        result = request.best_match_content_type()
        self.assertEqual("application/json", result)

    def test_content_type_from_accept_xml_json(self):
        request = wsgi.Request.blank('/tests/123')
        request.headers["Accept"] = "application/xml, application/json"
        result = request.best_match_content_type()
        self.assertEqual("application/json", result)

    def test_content_type_from_accept_json_xml_quality(self):
        request = wsgi.Request.blank('/tests/123')
        request.headers["Accept"] = ("application/json; q=0.3, "
                                     "application/xml; q=0.9")
        result = request.best_match_content_type()
        self.assertEqual("application/json", result)

    def test_content_type_accept_default(self):
        request = wsgi.Request.blank('/tests/123.unsupported')
        request.headers["Accept"] = "application/unsupported1"
        result = request.best_match_content_type()
        self.assertEqual("application/json", result)

    def test_language_accept_default(self):
        request = wsgi.Request.blank('/tests/123')
        request.headers["Accept-Language"] = "zz-ZZ,zz;q=0.8"
        result = request.best_match_language()
        self.assertIsNone(result)

    def test_language_accept_none(self):
        request = wsgi.Request.blank('/tests/123')
        result = request.best_match_language()
        self.assertIsNone(result)

    def test_best_match_language_expected(self):
        # If Accept-Language is a supported language, best_match_language()
        # returns it.
        self._set_expected_languages(all_locales=['it'])

        req = wsgi.Request.blank('/', headers={'Accept-Language': 'it'})
        self.assertEqual('it', req.best_match_language())

    def test_request_match_language_unexpected(self):
        # If Accept-Language is a language we do not support,
        # best_match_language() returns None.
        self._set_expected_languages(all_locales=['it'])

        req = wsgi.Request.blank('/', headers={'Accept-Language': 'unknown'})
        self.assertIsNone(req.best_match_language())

    @mock.patch.object(webob.acceptparse.AcceptLanguageValidHeader, 'lookup')
    def test_best_match_language_unknown(self, mock_lookup):
        # Test that we are actually invoking language negotiation by WebOb
        request = wsgi.Request.blank('/')
        accepted = 'unknown-lang'
        request.headers = {'Accept-Language': accepted}

        # Bug #1765748: see comment in code in the function under test
        # to understand why this is the correct return value for the
        # webob 1.8.x mock
        mock_lookup.return_value = 'fake_LANG'

        self.assertIsNone(request.best_match_language())
        mock_lookup.assert_called_once()

        # If Accept-Language is missing or empty, match should be None
        request.headers = {'Accept-Language': ''}
        self.assertIsNone(request.best_match_language())
        request.headers.pop('Accept-Language')
        self.assertIsNone(request.best_match_language())

    def test_http_error_response_codes(self):
        sample_id, member_id, tag_val, task_id = 'abc', '123', '1', '2'

        """Makes sure v2 unallowed methods return 405"""
        unallowed_methods = [
            ('/schemas/image', ['POST', 'PUT', 'DELETE', 'PATCH', 'HEAD']),
            ('/schemas/images', ['POST', 'PUT', 'DELETE', 'PATCH', 'HEAD']),
            ('/schemas/member', ['POST', 'PUT', 'DELETE', 'PATCH', 'HEAD']),
            ('/schemas/members', ['POST', 'PUT', 'DELETE', 'PATCH', 'HEAD']),
            ('/schemas/task', ['POST', 'PUT', 'DELETE', 'PATCH', 'HEAD']),
            ('/schemas/tasks', ['POST', 'PUT', 'DELETE', 'PATCH', 'HEAD']),
            ('/images', ['PUT', 'DELETE', 'PATCH', 'HEAD']),
            ('/images/%s' % sample_id, ['POST', 'PUT', 'HEAD']),
            ('/images/%s/file' % sample_id,
                ['POST', 'DELETE', 'PATCH', 'HEAD']),
            ('/images/%s/tags/%s' % (sample_id, tag_val),
                ['GET', 'POST', 'PATCH', 'HEAD']),
            ('/images/%s/members' % sample_id,
                ['PUT', 'DELETE', 'PATCH', 'HEAD']),
            ('/images/%s/members/%s' % (sample_id, member_id),
                ['POST', 'PATCH', 'HEAD']),
            ('/tasks', ['PUT', 'DELETE', 'PATCH', 'HEAD']),
            ('/tasks/%s' % task_id, ['POST', 'PUT', 'PATCH', 'HEAD']),
        ]
        api = test_utils.FakeAuthMiddleware(router_v2.API(routes.Mapper()))
        for uri, methods in unallowed_methods:
            for method in methods:
                req = webob.Request.blank(uri)
                req.method = method
                res = req.get_response(api)
                self.assertEqual(http.METHOD_NOT_ALLOWED, res.status_int)

        # Makes sure not implemented methods return 405
        req = webob.Request.blank('/schemas/image')
        req.method = 'NonexistentMethod'
        res = req.get_response(api)
        self.assertEqual(http.METHOD_NOT_ALLOWED, res.status_int)


class ResourceTest(test_utils.BaseTestCase):

    def test_get_action_args(self):
        env = {
            'wsgiorg.routing_args': [
                None,
                {
                    'controller': None,
                    'format': None,
                    'action': 'update',
                    'id': 12,
                },
            ],
        }

        expected = {'action': 'update', 'id': 12}
        actual = wsgi.Resource(None, None, None).get_action_args(env)

        self.assertEqual(expected, actual)

    def test_get_action_args_invalid_index(self):
        env = {'wsgiorg.routing_args': []}
        expected = {}
        actual = wsgi.Resource(None, None, None).get_action_args(env)
        self.assertEqual(expected, actual)

    def test_get_action_args_del_controller_error(self):
        actions = {'format': None,
                   'action': 'update',
                   'id': 12}
        env = {'wsgiorg.routing_args': [None, actions]}
        expected = {'action': 'update', 'id': 12}
        actual = wsgi.Resource(None, None, None).get_action_args(env)
        self.assertEqual(expected, actual)

    def test_get_action_args_del_format_error(self):
        actions = {'action': 'update', 'id': 12}
        env = {'wsgiorg.routing_args': [None, actions]}
        expected = {'action': 'update', 'id': 12}
        actual = wsgi.Resource(None, None, None).get_action_args(env)
        self.assertEqual(expected, actual)

    def test_dispatch(self):
        class Controller(object):
            def index(self, shirt, pants=None):
                return (shirt, pants)

        resource = wsgi.Resource(None, None, None)
        actual = resource.dispatch(Controller(), 'index', 'on', pants='off')
        expected = ('on', 'off')
        self.assertEqual(expected, actual)

    def test_dispatch_default(self):
        class Controller(object):
            def default(self, shirt, pants=None):
                return (shirt, pants)

        resource = wsgi.Resource(None, None, None)
        actual = resource.dispatch(Controller(), 'index', 'on', pants='off')
        expected = ('on', 'off')
        self.assertEqual(expected, actual)

    def test_dispatch_no_default(self):
        class Controller(object):
            def show(self, shirt, pants=None):
                return (shirt, pants)

        resource = wsgi.Resource(None, None, None)
        self.assertRaises(AttributeError, resource.dispatch, Controller(),
                          'index', 'on', pants='off')

    def test_dispatch_raises_bad_request(self):
        class FakeController(object):
            def index(self, shirt, pants=None):
                return (shirt, pants)
        resource = wsgi.Resource(FakeController(), None, None)

        def dispatch(self, obj, action, *args, **kwargs):
            raise exception.InvalidPropertyProtectionConfiguration()
        self.mock_object(wsgi.Resource, 'dispatch', dispatch)
        request = wsgi.Request.blank('/')
        self.assertRaises(webob.exc.HTTPBadRequest, resource.__call__,
                          request)

    def test_call(self):
        class FakeController(object):
            def index(self, shirt, pants=None):
                return (shirt, pants)

        resource = wsgi.Resource(FakeController(), None, None)

        def dispatch(self, obj, action, *args, **kwargs):
            if isinstance(obj, wsgi.JSONRequestDeserializer):
                return []
            if isinstance(obj, wsgi.JSONResponseSerializer):
                raise webob.exc.HTTPForbidden()

        self.mock_object(wsgi.Resource, 'dispatch', dispatch)

        request = wsgi.Request.blank('/')

        response = resource.__call__(request)

        self.assertIsInstance(response, webob.exc.HTTPForbidden)
        self.assertEqual(http.FORBIDDEN, response.status_code)

    def test_call_raises_exception(self):
        class FakeController(object):
            def index(self, shirt, pants=None):
                return (shirt, pants)

        resource = wsgi.Resource(FakeController(), None, None)

        def dispatch(self, obj, action, *args, **kwargs):
            raise Exception("test exception")

        self.mock_object(wsgi.Resource, 'dispatch', dispatch)

        request = wsgi.Request.blank('/')

        response = resource.__call__(request)

        self.assertIsInstance(response, webob.exc.HTTPInternalServerError)
        self.assertEqual(http.INTERNAL_SERVER_ERROR, response.status_code)

    @mock.patch.object(wsgi, 'translate_exception')
    def test_resource_call_error_handle_localized(self,
                                                  mock_translate_exception):
        class Controller(object):
            def delete(self, req, identity):
                raise webob.exc.HTTPBadRequest(explanation='Not Found')

        actions = {'action': 'delete', 'identity': 12}
        env = {'wsgiorg.routing_args': [None, actions]}
        request = wsgi.Request.blank('/tests/123', environ=env)
        message_es = 'No Encontrado'

        resource = wsgi.Resource(Controller(),
                                 wsgi.JSONRequestDeserializer(),
                                 None)
        translated_exc = webob.exc.HTTPBadRequest(message_es)
        mock_translate_exception.return_value = translated_exc

        e = self.assertRaises(webob.exc.HTTPBadRequest,
                              resource, request)
        self.assertEqual(message_es, str(e))

    @mock.patch.object(webob.acceptparse.AcceptLanguageValidHeader, 'lookup')
    @mock.patch.object(i18n, 'translate')
    def test_translate_exception(self, mock_translate, mock_lookup):
        mock_translate.return_value = 'No Encontrado'
        mock_lookup.return_value = 'de'

        req = wsgi.Request.blank('/tests/123')
        req.headers["Accept-Language"] = "de"

        e = webob.exc.HTTPNotFound(explanation='Not Found')
        e = wsgi.translate_exception(req, e)
        self.assertEqual('No Encontrado', e.explanation)

    def test_response_headers_encoded(self):
        # prepare environment
        for_openstack_comrades = (
            '\u0417\u0430 \u043e\u043f\u0435\u043d\u0441\u0442\u0435\u043a, '
            '\u0442\u043e\u0432\u0430\u0440\u0438\u0449\u0438')

        class FakeController(object):
            def index(self, shirt, pants=None):
                return (shirt, pants)

        class FakeSerializer(object):
            def index(self, response, result):
                response.headers['unicode_test'] = for_openstack_comrades

        # make request
        resource = wsgi.Resource(FakeController(), None, FakeSerializer())
        actions = {'action': 'index'}
        env = {'wsgiorg.routing_args': [None, actions]}
        request = wsgi.Request.blank('/tests/123', environ=env)
        response = resource.__call__(request)

        # ensure it has been encoded correctly
        value = response.headers['unicode_test']
        self.assertEqual(for_openstack_comrades, value)


class JSONResponseSerializerTest(test_utils.BaseTestCase):

    def test_to_json(self):
        fixture = {"key": "value"}
        expected = b'{"key": "value"}'
        actual = wsgi.JSONResponseSerializer().to_json(fixture)
        self.assertEqual(expected, actual)

    def test_to_json_with_date_format_value(self):
        fixture = {"date": datetime.datetime(1901, 3, 8, 2)}
        expected = b'{"date": "1901-03-08T02:00:00.000000"}'
        actual = wsgi.JSONResponseSerializer().to_json(fixture)
        self.assertEqual(expected, actual)

    def test_to_json_with_more_deep_format(self):
        fixture = {"is_public": True, "name": [{"name1": "test"}]}
        expected = {"is_public": True, "name": [{"name1": "test"}]}
        actual = wsgi.JSONResponseSerializer().to_json(fixture)
        actual = jsonutils.loads(actual)
        for k in expected:
            self.assertEqual(expected[k], actual[k])

    def test_to_json_with_set(self):
        fixture = set(["foo"])
        expected = b'["foo"]'
        actual = wsgi.JSONResponseSerializer().to_json(fixture)
        self.assertEqual(expected, actual)

    def test_default(self):
        fixture = {"key": "value"}
        response = webob.Response()
        wsgi.JSONResponseSerializer().default(response, fixture)
        self.assertEqual(http.OK, response.status_int)
        content_types = [h for h in response.headerlist
                         if h[0] == 'Content-Type']
        self.assertEqual(1, len(content_types))
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(b'{"key": "value"}', response.body)


class JSONRequestDeserializerTest(test_utils.BaseTestCase):

    def test_has_body_no_content_length(self):
        request = wsgi.Request.blank('/')
        request.method = 'POST'
        request.body = b'asdf'
        request.headers.pop('Content-Length')
        self.assertFalse(wsgi.JSONRequestDeserializer().has_body(request))

    def test_has_body_zero_content_length(self):
        request = wsgi.Request.blank('/')
        request.method = 'POST'
        request.body = b'asdf'
        request.headers['Content-Length'] = 0
        self.assertFalse(wsgi.JSONRequestDeserializer().has_body(request))

    def test_has_body_has_content_length(self):
        request = wsgi.Request.blank('/')
        request.method = 'POST'
        request.body = b'asdf'
        self.assertIn('Content-Length', request.headers)
        self.assertTrue(wsgi.JSONRequestDeserializer().has_body(request))

    def test_no_body_no_content_length(self):
        request = wsgi.Request.blank('/')
        self.assertFalse(wsgi.JSONRequestDeserializer().has_body(request))

    def test_from_json(self):
        fixture = '{"key": "value"}'
        expected = {"key": "value"}
        actual = wsgi.JSONRequestDeserializer().from_json(fixture)
        self.assertEqual(expected, actual)

    def test_from_json_malformed(self):
        fixture = 'kjasdklfjsklajf'
        self.assertRaises(webob.exc.HTTPBadRequest,
                          wsgi.JSONRequestDeserializer().from_json, fixture)

    def test_default_no_body(self):
        request = wsgi.Request.blank('/')
        actual = wsgi.JSONRequestDeserializer().default(request)
        expected = {}
        self.assertEqual(expected, actual)

    def test_default_with_body(self):
        request = wsgi.Request.blank('/')
        request.method = 'POST'
        request.body = b'{"key": "value"}'
        actual = wsgi.JSONRequestDeserializer().default(request)
        expected = {"body": {"key": "value"}}
        self.assertEqual(expected, actual)

    def test_has_body_has_transfer_encoding(self):
        self.assertTrue(self._check_transfer_encoding(
                        transfer_encoding='chunked'))

    def test_has_body_multiple_transfer_encoding(self):
        self.assertTrue(self._check_transfer_encoding(
                        transfer_encoding='chunked, gzip'))

    def test_has_body_invalid_transfer_encoding(self):
        self.assertFalse(self._check_transfer_encoding(
                         transfer_encoding='invalid', content_length=0))

    def test_has_body_invalid_transfer_encoding_no_content_len_and_body(self):
        self.assertFalse(self._check_transfer_encoding(
                         transfer_encoding='invalid', include_body=False))

    def test_has_body_invalid_transfer_encoding_no_content_len_but_body(self):
        self.assertTrue(self._check_transfer_encoding(
                        transfer_encoding='invalid', include_body=True))

    def test_has_body_invalid_transfer_encoding_with_content_length(self):
        self.assertTrue(self._check_transfer_encoding(
                        transfer_encoding='invalid', content_length=5))

    def test_has_body_valid_transfer_encoding_with_content_length(self):
        self.assertTrue(self._check_transfer_encoding(
                        transfer_encoding='chunked', content_length=1))

    def test_has_body_valid_transfer_encoding_without_content_length(self):
        self.assertTrue(self._check_transfer_encoding(
                        transfer_encoding='chunked'))

    def _check_transfer_encoding(self, transfer_encoding=None,
                                 content_length=None, include_body=True):
        request = wsgi.Request.blank('/')
        request.method = 'POST'
        if include_body:
            request.body = b'fake_body'
        request.headers['transfer-encoding'] = transfer_encoding
        if content_length is not None:
            request.headers['content-length'] = content_length

        return wsgi.JSONRequestDeserializer().has_body(request)

    def test_get_bind_addr_default_value(self):
        expected = ('0.0.0.0', '123456')
        actual = wsgi.get_bind_addr(default_port="123456")
        self.assertEqual(expected, actual)


class ServerTest(test_utils.BaseTestCase):
    @mock.patch.object(prefetcher, 'Prefetcher')
    def test_create_pool(self, mock_prefetcher):
        """Ensure the wsgi thread pool is an eventlet.greenpool.GreenPool."""
        actual = wsgi.Server(threads=1).create_pool()
        self.assertIsInstance(actual, eventlet.greenpool.GreenPool)

    @mock.patch.object(prefetcher, 'Prefetcher')
    @mock.patch.object(wsgi.Server, 'configure_socket')
    def test_reserved_stores_not_allowed(self, mock_configure_socket,
                                         mock_prefetcher):
        """Ensure the reserved stores are not allowed"""
        enabled_backends = {'os_glance_file_store': 'file'}
        self.config(enabled_backends=enabled_backends)
        server = wsgi.Server(threads=1, initialize_glance_store=True)
        self.assertRaises(RuntimeError, server.configure)

    @mock.patch.object(prefetcher, 'Prefetcher')
    @mock.patch.object(wsgi.Server, 'configure_socket')
    @mock.patch('glance.sqlite_migration.can_migrate_to_central_db')
    def test_http_keepalive(self, mock_migrate_db, mock_configure_socket,
                            mock_prefetcher):
        mock_migrate_db.return_value = False
        self.config(http_keepalive=False)
        self.config(workers=0)

        server = wsgi.Server(threads=1)
        server.sock = 'fake_socket'
        # mocking eventlet.wsgi server method to check it is called with
        # configured 'http_keepalive' value.
        with mock.patch.object(eventlet.wsgi,
                               'server') as mock_server:
            fake_application = "fake-application"
            server.start(fake_application, 0)
            server.wait()
            mock_server.assert_called_once_with('fake_socket',
                                                fake_application,
                                                log=server._logger,
                                                debug=False,
                                                custom_pool=server.pool,
                                                keepalive=False,
                                                socket_timeout=900)

    @mock.patch.object(prefetcher, 'Prefetcher')
    @mock.patch('glance.sqlite_migration.can_migrate_to_central_db')
    def test_number_of_workers_posix(self, mock_migrate_db, mock_prefetcher):
        """Ensure the number of workers matches num cpus limited to 8."""
        mock_migrate_db.return_value = False

        def pid():
            i = 1
            while True:
                i = i + 1
                yield i

        with mock.patch.object(os, 'fork') as mock_fork:
            with mock.patch('oslo_concurrency.processutils.get_worker_count',
                            return_value=4):
                mock_fork.side_effect = pid
                server = wsgi.Server()
                server.configure = mock.Mock()
                fake_application = "fake-application"
                server.start(fake_application, None)
                self.assertEqual(4, len(server.children))
            with mock.patch('oslo_concurrency.processutils.get_worker_count',
                            return_value=24):
                mock_fork.side_effect = pid
                server = wsgi.Server()
                server.configure = mock.Mock()
                fake_application = "fake-application"
                server.start(fake_application, None)
                self.assertEqual(8, len(server.children))
            mock_fork.side_effect = pid
            server = wsgi.Server()
            server.configure = mock.Mock()
            fake_application = "fake-application"
            server.start(fake_application, None)
            cpus = processutils.get_worker_count()
            expected_workers = cpus if cpus < 8 else 8
            self.assertEqual(expected_workers,
                             len(server.children))

    @mock.patch('glance.sqlite_migration.can_migrate_to_central_db')
    def test_invalid_staging_uri(self, mock_migrate_db):
        mock_migrate_db.return_value = False
        self.config(node_staging_uri='http://good.luck')
        server = wsgi.Server()
        with mock.patch.object(server, 'start_wsgi'):
            # Make sure a stating URI with an bad scheme will abort startup
            self.assertRaises(exception.GlanceException,
                              server.start, 'fake-application', 34567)

    @mock.patch('os.path.exists')
    @mock.patch('glance.sqlite_migration.can_migrate_to_central_db')
    def test_missing_staging_dir(self, mock_migrate_db, mock_exists):
        mock_migrate_db.return_value = False
        mock_exists.return_value = False
        server = wsgi.Server()
        with mock.patch.object(server, 'start_wsgi'):
            # Since we are mocking out start_wsgi, create a fake pool ourselves
            server.pool = mock.MagicMock()
            with mock.patch.object(wsgi, 'LOG') as mock_log:
                server.start('fake-application', 34567)
                mock_exists.assert_called_once_with('/tmp/staging/')
                # Make sure a missing staging directory will log a warning.
                mock_log.warning.assert_called_once_with(
                    'Import methods are enabled but staging directory '
                    '%(path)s does not exist; Imports will fail!',
                    {'path': '/tmp/staging/'})


class TestHelpers(test_utils.BaseTestCase):

    def test_headers_are_unicode(self):
        """
        Verifies that the headers returned by conversion code are unicode.

        Headers are passed via http in non-testing mode, which automatically
        converts them to unicode. Verifying that the method does the
        conversion proves that we aren't passing data that works in tests
        but will fail in production.
        """
        fixture = {'name': 'fake public image',
                   'is_public': True,
                   'size': 19,
                   'location': "file:///tmp/glance-tests/2",
                   'properties': {'distro': 'Ubuntu 10.04 LTS'}}
        headers = utils.image_meta_to_http_headers(fixture)
        for k, v in headers.items():
            self.assertIsInstance(v, str)

    def test_data_passed_properly_through_headers(self):
        """
        Verifies that data is the same after being passed through headers
        """
        fixture = {'is_public': True,
                   'deleted': False,
                   'name': None,
                   'size': 19,
                   'location': "file:///tmp/glance-tests/2",
                   'properties': {'distro': 'Ubuntu 10.04 LTS'}}
        headers = utils.image_meta_to_http_headers(fixture)

        class FakeResponse(object):
            pass

        response = FakeResponse()
        response.headers = headers
        result = utils.get_image_meta_from_headers(response)
        for k, v in fixture.items():
            if v is not None:
                self.assertEqual(v, result[k])
            else:
                self.assertNotIn(k, result)


class GetSocketTestCase(test_utils.BaseTestCase):

    def setUp(self):
        super(GetSocketTestCase, self).setUp()
        self.useFixture(fixtures.MonkeyPatch(
            "glance.common.wsgi.get_bind_addr",
            lambda x: ('192.168.0.13', 1234)))
        addr_info_list = [(2, 1, 6, '', ('192.168.0.13', 80)),
                          (2, 2, 17, '', ('192.168.0.13', 80)),
                          (2, 3, 0, '', ('192.168.0.13', 80))]
        self.useFixture(fixtures.MonkeyPatch(
            "glance.common.wsgi.socket.getaddrinfo",
            lambda *x: addr_info_list))
        self.useFixture(fixtures.MonkeyPatch(
            "glance.common.wsgi.time.time",
            mock.Mock(side_effect=[0, 1, 5, 10, 20, 35])))
        wsgi.CONF.tcp_keepidle = 600

    @mock.patch.object(prefetcher, 'Prefetcher')
    def test_correct_configure_socket(self, mock_prefetcher):
        mock_socket = mock.Mock()
        self.useFixture(fixtures.MonkeyPatch(
            'glance.common.wsgi.eventlet.listen',
            lambda *x, **y: mock_socket))
        server = wsgi.Server()
        server.default_port = 1234
        server.configure_socket()
        self.assertIn(mock.call.setsockopt(
            socket.SOL_SOCKET,
            socket.SO_REUSEADDR,
            1), mock_socket.mock_calls)
        self.assertIn(mock.call.setsockopt(
            socket.SOL_SOCKET,
            socket.SO_KEEPALIVE,
            1), mock_socket.mock_calls)
        if hasattr(socket, 'TCP_KEEPIDLE'):
            self.assertIn(mock.call.setsockopt(
                socket.IPPROTO_TCP,
                socket.TCP_KEEPIDLE,
                wsgi.CONF.tcp_keepidle), mock_socket.mock_calls)

    def test_get_socket_with_bind_problems(self):
        self.useFixture(fixtures.MonkeyPatch(
            'glance.common.wsgi.eventlet.listen',
            mock.Mock(side_effect=(
                [wsgi.socket.error(socket.errno.EADDRINUSE)] * 3 + [None]))))

        self.assertRaises(RuntimeError, wsgi.get_socket, 1234)

    def test_get_socket_with_unexpected_socket_errno(self):
        self.useFixture(fixtures.MonkeyPatch(
            'glance.common.wsgi.eventlet.listen',
            mock.Mock(side_effect=wsgi.socket.error(socket.errno.ENOMEM))))
        self.assertRaises(wsgi.socket.error, wsgi.get_socket, 1234)


def _cleanup_uwsgi():
    wsgi.uwsgi = None


class Test_UwsgiChunkedFile(test_utils.BaseTestCase):

    def test_read_no_data(self):
        reader = wsgi._UWSGIChunkFile()
        wsgi.uwsgi = mock.MagicMock()
        self.addCleanup(_cleanup_uwsgi)

        def fake_read():
            return None

        wsgi.uwsgi.chunked_read = fake_read
        out = reader.read()
        self.assertEqual(out, b'')

    def test_read_data_no_length(self):
        reader = wsgi._UWSGIChunkFile()
        wsgi.uwsgi = mock.MagicMock()
        self.addCleanup(_cleanup_uwsgi)

        values = iter([b'a', b'b', b'c', None])

        def fake_read():
            return next(values)

        wsgi.uwsgi.chunked_read = fake_read
        out = reader.read()
        self.assertEqual(out, b'abc')

    def test_read_zero_length(self):
        reader = wsgi._UWSGIChunkFile()
        self.assertEqual(b'', reader.read(length=0))

    def test_read_data_length(self):
        reader = wsgi._UWSGIChunkFile()
        wsgi.uwsgi = mock.MagicMock()
        self.addCleanup(_cleanup_uwsgi)

        values = iter([b'a', b'b', b'c', None])

        def fake_read():
            return next(values)

        wsgi.uwsgi.chunked_read = fake_read
        out = reader.read(length=2)
        self.assertEqual(out, b'ab')

    def test_read_data_negative_length(self):
        reader = wsgi._UWSGIChunkFile()
        wsgi.uwsgi = mock.MagicMock()
        self.addCleanup(_cleanup_uwsgi)

        values = iter([b'a', b'b', b'c', None])

        def fake_read():
            return next(values)

        wsgi.uwsgi.chunked_read = fake_read
        out = reader.read(length=-2)
        self.assertEqual(out, b'abc')



# Copyright 2021 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg
from oslo_limit import exception as ol_exc
from oslo_limit import limit
from oslo_log import log as logging
from oslo_utils import units

from glance.common import exception
from glance.db.sqlalchemy import api as db
from glance.i18n import _LE

CONF = cfg.CONF
CONF.import_opt('use_keystone_limits', 'glance.common.config')
LOG = logging.getLogger(__name__)
limit.opts.register_opts(CONF)

QUOTA_IMAGE_SIZE_TOTAL = 'image_size_total'
QUOTA_IMAGE_STAGING_TOTAL = 'image_stage_total'
QUOTA_IMAGE_COUNT_TOTAL = 'image_count_total'
QUOTA_IMAGE_COUNT_UPLOADING = 'image_count_uploading'


def _enforce_some(context, project_id, quota_value_fns, deltas):
    """Helper method to enforce a set of quota values.

    :param context: The RequestContext
    :param project_id: The project_id of the tenant being checked
    :param get_value_fns: A mapping of quota names to functions that will be
                          called with no arguments to return the numerical
                          value representing current usage.
    :param deltas: A mapping of quota names to the amount of resource being
                   requested for each (to be added to the current usage before
                   determining if over-quota).
    :raises: exception.LimitExceeded if the current usage is over the defined
             limit.
    :returns: None if the tenant is not currently over their quota.
    """
    if not CONF.use_keystone_limits:
        return

    def callback(project_id, resource_names):
        return {name: quota_value_fns[name]()
                for name in resource_names}

    enforcer = limit.Enforcer(callback)
    try:
        enforcer.enforce(project_id,
                         {quota_name: deltas.get(quota_name, 0)
                          for quota_name in quota_value_fns})
    except ol_exc.ProjectOverLimit as e:
        raise exception.LimitExceeded(body=str(e))
    except ol_exc.SessionInitError as e:
        LOG.error(_LE('Failed to initialize oslo_limit, likely due to '
                      'incorrect or insufficient configuration: %(err)s'),
                  {'err': str(e)})
        # We could just raise LimitExceeded here, but a 500 is
        # appropriate for incorrect server-side configuration, so we
        # re-raise here after the above error message to make sure we
        # are noticed.
        raise


def _enforce_one(context, project_id, quota_name, get_value_fn, delta=0):
    """Helper method to enforce a single named quota value.

    :param context: The RequestContext
    :param project_id: The project_id of the tenant being checked
    :param quota_name: One of the quota names defined above
    :param get_value_fn: A function that will be called with no arguments to
                         return the numerical value representing current usage.
    :param delta: The amount of resource being requested (to be added to the
                  current usage before determining if over-quota).
    :raises: exception.LimitExceeded if the current usage is over the defined
             limit.
    :returns: None if the tenant is not currently over their quota.
    """

    return _enforce_some(context, project_id,
                         {quota_name: get_value_fn},
                         {quota_name: delta})


def enforce_image_size_total(context, project_id, delta=0):
    """Enforce the image_size_total quota.

    This enforces the total image size quota for the supplied project_id.
    """
    _enforce_one(
        context, project_id, QUOTA_IMAGE_SIZE_TOTAL,
        lambda: db.user_get_storage_usage(context, project_id) // units.Mi,
        delta=delta)


def enforce_image_staging_total(context, project_id, delta=0):
    """Enforce the image_stage_total quota.

    This enforces the total size of all images stored in staging areas
    for the supplied project_id.
    """
    _enforce_one(
        context, project_id, QUOTA_IMAGE_STAGING_TOTAL,
        lambda: db.user_get_staging_usage(context, project_id) // units.Mi,
        delta=delta)


def enforce_image_count_total(context, project_id):
    """Enforce the image_count_total quota.

    This enforces the total count of non-deleted images owned by the
    supplied project_id.
    """
    _enforce_one(
        context, project_id, QUOTA_IMAGE_COUNT_TOTAL,
        lambda: db.user_get_image_count(context, project_id),
        delta=1)


def enforce_image_count_uploading(context, project_id):
    """Enforce the image_count_uploading quota.

    This enforces the total count of images in any state of upload by
    the supplied project_id.

    :param delta: This defaults to one, but should be zero when checking
                  an operation on an image that already counts against this
                  quota (i.e. a stage operation of an existing queue image).
    """
    _enforce_one(
        context, project_id, QUOTA_IMAGE_COUNT_UPLOADING,
        lambda: db.user_get_uploading_count(context, project_id),
        delta=0)


def get_usage(context, project_id=None):
    if not CONF.use_keystone_limits:
        return {}

    if not project_id:
        project_id = context.project_id

    usages = {
        QUOTA_IMAGE_SIZE_TOTAL: lambda: db.user_get_storage_usage(
            context, project_id) // units.Mi,
        QUOTA_IMAGE_STAGING_TOTAL: lambda: db.user_get_staging_usage(
            context, project_id) // units.Mi,
        QUOTA_IMAGE_COUNT_TOTAL: lambda: db.user_get_image_count(
            context, project_id),
        QUOTA_IMAGE_COUNT_UPLOADING: lambda: db.user_get_uploading_count(
            context, project_id),
    }

    def callback(project_id, resource_names):
        return {name: usages[name]()
                for name in resource_names}

    enforcer = limit.Enforcer(callback)
    return enforcer.calculate_usage(project_id, list(usages.keys()))



# Copyright 2021 Red Hat, Inc
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

import webob.exc

from glance.api.v2 import policy
from glance.common import exception
from glance.tests import utils


class APIPolicyBase(utils.BaseTestCase):
    def setUp(self):
        super(APIPolicyBase, self).setUp()
        self.enforcer = mock.MagicMock()
        self.context = mock.MagicMock()
        self.policy = policy.APIPolicyBase(self.context,
                                           enforcer=self.enforcer)

    def test_enforce(self):
        # Enforce passes
        self.policy._enforce('fake_rule')
        self.enforcer.enforce.assert_called_once_with(
            self.context,
            'fake_rule',
            mock.ANY)

        # Make sure that Forbidden gets caught and translated
        self.enforcer.enforce.side_effect = exception.Forbidden
        self.assertRaises(webob.exc.HTTPForbidden,
                          self.policy._enforce, 'fake_rule')

        # Any other exception comes straight through
        self.enforcer.enforce.side_effect = exception.ImageNotFound
        self.assertRaises(exception.ImageNotFound,
                          self.policy._enforce, 'fake_rule')

    def test_check(self):
        # Check passes
        self.assertTrue(self.policy.check('_enforce', 'fake_rule'))

        # Check fails
        self.enforcer.enforce.side_effect = exception.Forbidden
        self.assertFalse(self.policy.check('_enforce', 'fake_rule'))

    def test_check_is_image_mutable(self):
        context = mock.MagicMock()
        image = mock.MagicMock()

        # Admin always wins
        context.is_admin = True
        context.owner = 'someuser'
        self.assertIsNone(policy.check_is_image_mutable(context, image))

        # Image has no owner is never mutable by non-admins
        context.is_admin = False
        image.owner = None
        self.assertRaises(exception.Forbidden,
                          policy.check_is_image_mutable,
                          context, image)

        # Not owner is not mutable
        image.owner = 'someoneelse'
        self.assertRaises(exception.Forbidden,
                          policy.check_is_image_mutable,
                          context, image)

        # No project in context means not mutable
        image.owner = 'someoneelse'
        context.owner = None
        self.assertRaises(exception.Forbidden,
                          policy.check_is_image_mutable,
                          context, image)

        # Context matches image owner is mutable
        image.owner = 'someuser'
        context.owner = 'someuser'
        self.assertIsNone(policy.check_is_image_mutable(context, image))


class APIImagePolicy(APIPolicyBase):
    def setUp(self):
        super(APIImagePolicy, self).setUp()
        self.image = mock.MagicMock()
        self.policy = policy.ImageAPIPolicy(self.context, self.image,
                                            enforcer=self.enforcer)

    def test_enforce(self):
        self.assertRaises(webob.exc.HTTPNotFound,
                          super(APIImagePolicy, self).test_enforce)

    @mock.patch('glance.api.policy._enforce_image_visibility')
    def test_enforce_visibility(self, mock_enf):
        # Visibility passes
        self.policy._enforce_visibility('something')
        mock_enf.assert_called_once_with(self.enforcer,
                                         self.context,
                                         'something',
                                         mock.ANY)

        # Make sure that Forbidden gets caught and translated
        mock_enf.side_effect = exception.Forbidden
        self.assertRaises(webob.exc.HTTPForbidden,
                          self.policy._enforce_visibility, 'something')

        # Any other exception comes straight through
        mock_enf.side_effect = exception.ImageNotFound
        self.assertRaises(exception.ImageNotFound,
                          self.policy._enforce_visibility, 'something')

    def test_update_property(self):
        with mock.patch.object(self.policy, '_enforce') as mock_enf:
            self.policy.update_property('foo', None)
            mock_enf.assert_called_once_with('modify_image')

        with mock.patch.object(self.policy, '_enforce_visibility') as mock_enf:
            self.policy.update_property('visibility', 'foo')
            mock_enf.assert_called_once_with('foo')

    def test_update_locations(self):
        self.policy.update_locations()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'set_image_location',
                                                      mock.ANY)

    def test_delete_locations(self):
        self.policy.delete_locations()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'delete_image_location',
                                                      mock.ANY)

    def test_delete_locations_falls_back_to_legacy(self):
        self.config(enforce_new_defaults=False, group='oslo_policy')
        self.config(enforce_scope=False, group='oslo_policy')

        # As admin, image is mutable even if owner does not match
        self.context.is_admin = True
        self.context.owner = 'someuser'
        self.image.owner = 'someotheruser'
        self.policy.delete_locations()

        # As non-admin, owner matches, so we're good
        self.context.is_admin = False
        self.context.owner = 'someuser'
        self.image.owner = 'someuser'
        self.policy.delete_locations()

        # If owner does not match, we fail
        self.image.owner = 'someotheruser'
        self.assertRaises(exception.Forbidden,
                          self.policy.delete_locations)

        # Make sure we are checking the legacy handler
        with mock.patch('glance.api.v2.policy.check_is_image_mutable') as m:
            self.policy.delete_locations()
            m.assert_called_once_with(self.context, self.image)

        # Make sure we are not checking it if enforce_new_defaults=True and
        # enforce_scope=True
        self.config(enforce_new_defaults=True, group='oslo_policy')
        self.config(enforce_scope=True, group='oslo_policy')
        with mock.patch('glance.api.v2.policy.check_is_image_mutable') as m:
            self.policy.delete_locations()
            self.assertFalse(m.called)

    def test_get_image_location(self):
        self.policy.get_image_location()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'get_image_location',
                                                      mock.ANY)

    def test_enforce_exception_behavior(self):
        with mock.patch.object(self.policy.enforcer, 'enforce') as mock_enf:
            # First make sure we can update if allowed
            self.policy.update_property('foo', None)
            self.assertTrue(mock_enf.called)

            # Make sure that if modify_image and get_image both return
            # Forbidden then we should get NotFound. This is because
            # we are not allowed to delete the image, nor see that it
            # even exists.
            mock_enf.reset_mock()
            mock_enf.side_effect = exception.Forbidden
            self.assertRaises(webob.exc.HTTPNotFound,
                              self.policy.update_property, 'foo', None)
            # Make sure we checked modify_image, and then get_image.
            mock_enf.assert_has_calls([
                mock.call(mock.ANY, 'modify_image', mock.ANY),
                mock.call(mock.ANY, 'get_image', mock.ANY)])

            # Make sure that if modify_image is disallowed, but
            # get_image is allowed, that we get Forbidden. This is
            # because we are allowed to see the image, but not modify
            # it, so 403 indicates that without confusing the user and
            # returning "not found" for an image they are able to GET.
            mock_enf.reset_mock()
            mock_enf.side_effect = [exception.Forbidden, lambda *a: None]
            self.assertRaises(webob.exc.HTTPForbidden,
                              self.policy.update_property, 'foo', None)
            # Make sure we checked modify_image, and then get_image.
            mock_enf.assert_has_calls([
                mock.call(mock.ANY, 'modify_image', mock.ANY),
                mock.call(mock.ANY, 'get_image', mock.ANY)])

    def test_get_image(self):
        self.policy.get_image()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'get_image',
                                                      mock.ANY)

    def test_get_images(self):
        self.policy.get_images()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'get_images',
                                                      mock.ANY)

    def test_add_image(self):
        generic_target = {'project_id': self.context.project_id,
                          'owner': self.context.project_id,
                          'visibility': 'private'}
        self.policy = policy.ImageAPIPolicy(self.context, {},
                                            enforcer=self.enforcer)
        self.policy.add_image()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'add_image',
                                                      generic_target)

    def test_add_image_falls_back_to_legacy(self):
        self.config(enforce_new_defaults=False, group='oslo_policy')
        self.config(enforce_scope=False, group='oslo_policy')

        self.context.is_admin = False
        self.policy = policy.ImageAPIPolicy(self.context, {'owner': 'else'},
                                            enforcer=self.enforcer)
        self.assertRaises(exception.Forbidden, self.policy.add_image)

        # Make sure we're calling the legacy handler if secure_rbac is False
        with mock.patch('glance.api.v2.policy.check_admin_or_same_owner') as m:
            self.policy.add_image()
            m.assert_called_once_with(self.context, {'project_id': 'else',
                                                     'owner': 'else',
                                                     'visibility': 'private'})

        # Make sure we are not calling the legacy handler if
        # secure_rbac is being used. We won't fail the check because
        # our enforcer is a mock, just make sure we don't call that handler.
        self.config(enforce_new_defaults=True, group='oslo_policy')
        self.config(enforce_scope=True, group='oslo_policy')
        with mock.patch('glance.api.v2.policy.check_admin_or_same_owner') as m:
            self.policy.add_image()
            m.assert_not_called()

    def test_add_image_translates_owner_failure(self):
        self.policy = policy.ImageAPIPolicy(self.context, {'owner': 'else'},
                                            enforcer=self.enforcer)
        # Make sure add_image works with no exception
        self.policy.add_image()

        # Make sure we don't get in the way of any other exceptions
        self.enforcer.enforce.side_effect = exception.Duplicate
        self.assertRaises(exception.Duplicate, self.policy.add_image)

        # If the exception is HTTPForbidden and the owner differs,
        # make sure we get the proper message translation
        self.enforcer.enforce.side_effect = webob.exc.HTTPForbidden('original')
        exc = self.assertRaises(webob.exc.HTTPForbidden, self.policy.add_image)
        self.assertIn('You are not permitted to create images owned by',
                      str(exc))

        # If the owner does not differ, make sure we get the original reason
        self.policy = policy.ImageAPIPolicy(self.context, {},
                                            enforcer=self.enforcer)
        exc = self.assertRaises(webob.exc.HTTPForbidden, self.policy.add_image)
        self.assertIn('original', str(exc))

    def test_delete_image(self):
        self.policy.delete_image()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'delete_image',
                                                      mock.ANY)

    def test_delete_image_falls_back_to_legacy(self):
        self.config(enforce_new_defaults=False, group='oslo_policy')
        self.config(enforce_scope=False, group='oslo_policy')
        # As admin, image is mutable even if owner does not match
        self.context.is_admin = True
        self.context.owner = 'someuser'
        self.image.owner = 'someotheruser'
        self.policy.delete_image()

        # As non-admin, owner matches, so we're good
        self.context.is_admin = False
        self.context.owner = 'someuser'
        self.image.owner = 'someuser'
        self.policy.delete_image()

        # If owner does not match, we fail
        self.image.owner = 'someotheruser'
        self.assertRaises(exception.Forbidden,
                          self.policy.delete_image)

        # Make sure we are checking the legacy handler
        with mock.patch('glance.api.v2.policy.check_is_image_mutable') as m:
            self.policy.delete_image()
            m.assert_called_once_with(self.context, self.image)

        # Make sure we are not checking it if enforce_new_defaults=True and
        # enforce_scope=True
        self.config(enforce_new_defaults=True, group='oslo_policy')
        self.config(enforce_scope=True, group='oslo_policy')
        with mock.patch('glance.api.v2.policy.check_is_image_mutable') as m:
            self.policy.delete_image()
            self.assertFalse(m.called)

    def test_upload_image(self):
        self.policy.upload_image()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'upload_image',
                                                      mock.ANY)

    def test_upload_image_falls_back_to_legacy(self):
        self.config(enforce_new_defaults=False, group='oslo_policy')
        self.config(enforce_scope=False, group='oslo_policy')

        # As admin, image is mutable even if owner does not match
        self.context.is_admin = True
        self.context.owner = 'someuser'
        self.image.owner = 'someotheruser'
        self.policy.upload_image()

        # As non-admin, owner matches, so we're good
        self.context.is_admin = False
        self.context.owner = 'someuser'
        self.image.owner = 'someuser'
        self.policy.upload_image()

        # If owner does not match, we fail
        self.image.owner = 'someotheruser'
        self.assertRaises(exception.Forbidden,
                          self.policy.upload_image)

        # Make sure we are checking the legacy handler
        with mock.patch('glance.api.v2.policy.check_is_image_mutable') as m:
            self.policy.upload_image()
            m.assert_called_once_with(self.context, self.image)

        # Make sure we are not checking it if enforce_new_defaults=True and
        # enforce_scope=True
        self.config(enforce_new_defaults=True, group='oslo_policy')
        self.config(enforce_scope=True, group='oslo_policy')
        with mock.patch('glance.api.v2.policy.check_is_image_mutable') as m:
            self.policy.upload_image()
            self.assertFalse(m.called)

    def test_download_image(self):
        self.policy.download_image()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'download_image',
                                                      mock.ANY)

    def test_modify_image(self):
        self.policy.modify_image()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'modify_image',
                                                      mock.ANY)

    def test_modify_image_falls_back_to_legacy(self):
        self.config(enforce_new_defaults=False, group='oslo_policy')
        self.config(enforce_scope=False, group='oslo_policy')

        # As admin, image is mutable even if owner does not match
        self.context.is_admin = True
        self.context.owner = 'someuser'
        self.image.owner = 'someotheruser'
        self.policy.modify_image()

        # As non-admin, owner matches, so we're good
        self.context.is_admin = False
        self.context.owner = 'someuser'
        self.image.owner = 'someuser'
        self.policy.modify_image()

        # If owner does not match, we fail
        self.image.owner = 'someotheruser'
        self.assertRaises(exception.Forbidden,
                          self.policy.modify_image)

        # Make sure we are checking the legacy handler
        with mock.patch('glance.api.v2.policy.check_is_image_mutable') as m:
            self.policy.modify_image()
            m.assert_called_once_with(self.context, self.image)

        # Make sure we are not checking it if enforce_new_defaults=True and
        # enforce_scope=True
        self.config(enforce_new_defaults=True, group='oslo_policy')
        self.config(enforce_scope=True, group='oslo_policy')
        with mock.patch('glance.api.v2.policy.check_is_image_mutable') as m:
            self.policy.modify_image()
            self.assertFalse(m.called)

    def test_deactivate_image(self):
        self.policy.deactivate_image()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'deactivate',
                                                      mock.ANY)

    def test_deactivate_image_falls_back_to_legacy(self):
        self.config(enforce_new_defaults=False, group='oslo_policy')
        self.config(enforce_scope=False, group='oslo_policy')

        # As admin, image is mutable even if owner does not match
        self.context.is_admin = True
        self.context.owner = 'someuser'
        self.image.owner = 'someotheruser'
        self.policy.deactivate_image()

        # As non-admin, owner matches, so we're good
        self.context.is_admin = False
        self.context.owner = 'someuser'
        self.image.owner = 'someuser'
        self.policy.delete_image()

        # If owner does not match, we fail
        self.image.owner = 'someotheruser'
        self.assertRaises(exception.Forbidden,
                          self.policy.deactivate_image)

        # Make sure we are checking the legacy handler
        with mock.patch('glance.api.v2.policy.check_is_image_mutable') as m:
            self.policy.deactivate_image()
            m.assert_called_once_with(self.context, self.image)

        # Make sure we are not checking it if enforce_new_defaults=Truei and
        # enforce_scope=True
        self.config(enforce_new_defaults=True, group='oslo_policy')
        self.config(enforce_scope=True, group='oslo_policy')
        with mock.patch('glance.api.v2.policy.check_is_image_mutable') as m:
            self.policy.deactivate_image()
            self.assertFalse(m.called)

    def test_reactivate_image(self):
        self.policy.reactivate_image()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'reactivate',
                                                      mock.ANY)

    def test_reactivate_image_falls_back_to_legacy(self):
        self.config(enforce_new_defaults=False, group='oslo_policy')
        self.config(enforce_scope=False, group='oslo_policy')

        # As admin, image is mutable even if owner does not match
        self.context.is_admin = True
        self.context.owner = 'someuser'
        self.image.owner = 'someotheruser'
        self.policy.reactivate_image()

        # As non-admin, owner matches, so we're good
        self.context.is_admin = False
        self.context.owner = 'someuser'
        self.image.owner = 'someuser'
        self.policy.delete_image()

        # If owner does not match, we fail
        self.image.owner = 'someotheruser'
        self.assertRaises(exception.Forbidden,
                          self.policy.reactivate_image)

        # Make sure we are checking the legacy handler
        with mock.patch('glance.api.v2.policy.check_is_image_mutable') as m:
            self.policy.reactivate_image()
            m.assert_called_once_with(self.context, self.image)

        # Make sure we are not checking it if enforce_new_defaults=True and
        # enforce_scope=True
        self.config(enforce_new_defaults=True, group='oslo_policy')
        self.config(enforce_scope=True, group='oslo_policy')
        with mock.patch('glance.api.v2.policy.check_is_image_mutable') as m:
            self.policy.reactivate_image()
            self.assertFalse(m.called)

    def test_copy_image(self):
        self.policy.copy_image()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'copy_image',
                                                      mock.ANY)


class TestMetadefAPIPolicy(APIPolicyBase):
    def setUp(self):
        super(TestMetadefAPIPolicy, self).setUp()
        self.enforcer = mock.MagicMock()
        self.md_resource = mock.MagicMock()
        self.context = mock.MagicMock()
        self.policy = policy.MetadefAPIPolicy(self.context, self.md_resource,
                                              enforcer=self.enforcer)

    def test_enforce(self):
        self.assertRaises(webob.exc.HTTPNotFound,
                          super(TestMetadefAPIPolicy, self).test_enforce)

    def test_get_metadef_namespace(self):
        self.policy.get_metadef_namespace()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'get_metadef_namespace',
                                                      mock.ANY)

    def test_get_metadef_namespaces(self):
        self.policy.get_metadef_namespaces()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'get_metadef_namespaces',
                                                      mock.ANY)

    def test_add_metadef_namespace(self):
        self.policy.add_metadef_namespace()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'add_metadef_namespace',
                                                      mock.ANY)

    def test_modify_metadef_namespace(self):
        self.policy.modify_metadef_namespace()
        self.enforcer.enforce.assert_called_once_with(
            self.context, 'modify_metadef_namespace', mock.ANY)

    def test_delete_metadef_namespace(self):
        self.policy.delete_metadef_namespace()
        self.enforcer.enforce.assert_called_once_with(
            self.context, 'delete_metadef_namespace', mock.ANY)

    def test_get_metadef_objects(self):
        self.policy.get_metadef_objects()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'get_metadef_objects',
                                                      mock.ANY)

    def test_get_metadef_object(self):
        self.policy.get_metadef_object()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'get_metadef_object',
                                                      mock.ANY)

    def test_add_metadef_object(self):
        self.policy.add_metadef_object()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'add_metadef_object',
                                                      mock.ANY)

    def test_modify_metadef_object(self):
        self.policy.modify_metadef_object()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'modify_metadef_object',
                                                      mock.ANY)

    def test_delete_metadef_object(self):
        self.policy.delete_metadef_object()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'delete_metadef_object',
                                                      mock.ANY)

    def test_add_metadef_tag(self):
        self.policy.add_metadef_tag()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'add_metadef_tag',
                                                      mock.ANY)

    def test_add_metadef_tags(self):
        self.policy.add_metadef_tags()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'add_metadef_tags',
                                                      mock.ANY)

    def test_get_metadef_tags(self):
        self.policy.get_metadef_tags()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'get_metadef_tags',
                                                      mock.ANY)

    def test_get_metadef_tag(self):
        self.policy.get_metadef_tag()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'get_metadef_tag',
                                                      mock.ANY)

    def modify_metadef_tag(self):
        self.policy.modify_metadef_tag()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'modify_metadef_tag',
                                                      mock.ANY)

    def test_delete_metadef_tags(self):
        self.policy.delete_metadef_tags()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'delete_metadef_tags',
                                                      mock.ANY)

    def test_delete_metadef_tag(self):
        self.policy.delete_metadef_tag()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'delete_metadef_tag',
                                                      mock.ANY)

    def test_add_metadef_property(self):
        self.policy.add_metadef_property()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'add_metadef_property',
                                                      mock.ANY)

    def test_get_metadef_properties(self):
        self.policy.get_metadef_properties()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'get_metadef_properties',
                                                      mock.ANY)

    def test_get_metadef_property(self):
        self.policy.get_metadef_property()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'get_metadef_property',
                                                      mock.ANY)

    def test_modify_metadef_property(self):
        self.policy.modify_metadef_property()
        self.enforcer.enforce.assert_called_once_with(
            self.context, 'modify_metadef_property', mock.ANY)

    def test_remove_metadef_property(self):
        self.policy.remove_metadef_property()
        self.enforcer.enforce.assert_called_once_with(
            self.context, 'remove_metadef_property', mock.ANY)

    def test_add_metadef_resource_type_association(self):
        self.policy.add_metadef_resource_type_association()
        self.enforcer.enforce.assert_called_once_with(
            self.context, 'add_metadef_resource_type_association', mock.ANY)

    def test_list_metadef_resource_types(self):
        self.policy.list_metadef_resource_types()
        self.enforcer.enforce.assert_called_once_with(
            self.context, 'list_metadef_resource_types', mock.ANY)

    def test_enforce_exception_behavior(self):
        with mock.patch.object(self.policy.enforcer, 'enforce') as mock_enf:
            # First make sure we can update if allowed
            self.policy.modify_metadef_namespace()
            self.assertTrue(mock_enf.called)

            # Make sure that if modify_metadef_namespace and
            # get_metadef_namespace both return Forbidden then we
            # should get NotFound. This is because we are not allowed
            # to modify the namespace, nor see that it even exists.
            mock_enf.reset_mock()
            mock_enf.side_effect = exception.Forbidden
            self.assertRaises(webob.exc.HTTPNotFound,
                              self.policy.modify_metadef_namespace)
            # Make sure we checked modify_metadef_namespace, and then
            # get_metadef_namespace.
            mock_enf.assert_has_calls([
                mock.call(mock.ANY, 'modify_metadef_namespace', mock.ANY),
                mock.call(mock.ANY, 'get_metadef_namespace', mock.ANY)])

            # Make sure that if modify_metadef_namespace is disallowed, but
            # get_metadef_namespace is allowed, that we get Forbidden. This is
            # because we are allowed to see the namespace, but not modify
            # it, so 403 indicates that without confusing the user and
            # returning "not found" for a namespace they are able to GET.
            mock_enf.reset_mock()
            mock_enf.side_effect = [exception.Forbidden, lambda *a: None]
            self.assertRaises(webob.exc.HTTPForbidden,
                              self.policy.modify_metadef_namespace)
            # Make sure we checked modify_metadef_namespace, and then
            # get_metadef_namespace.
            mock_enf.assert_has_calls([
                mock.call(mock.ANY, 'modify_metadef_namespace', mock.ANY),
                mock.call(mock.ANY, 'get_metadef_namespace', mock.ANY)])

    def test_get_metadef_resource_type(self):
        self.policy.get_metadef_resource_type()
        self.enforcer.enforce.assert_called_once_with(
            self.context, 'get_metadef_resource_type', mock.ANY)

    def test_remove_metadef_resource_type_association(self):
        self.policy.remove_metadef_resource_type_association()
        self.enforcer.enforce.assert_called_once_with(
            self.context, 'remove_metadef_resource_type_association', mock.ANY)


class TestMemberAPIPolicy(utils.BaseTestCase):
    def setUp(self):
        super(TestMemberAPIPolicy, self).setUp()
        self.enforcer = mock.MagicMock()
        self.image = mock.MagicMock()
        self.context = mock.MagicMock()
        self.policy = policy.MemberAPIPolicy(self.context, self.image,
                                             enforcer=self.enforcer)

    def test_enforce(self):
        # Enforce passes
        self.policy._enforce('fake_rule')
        expected_calls = [
            mock.call(self.context, 'get_image', mock.ANY),
            mock.call(self.context, 'fake_rule', mock.ANY)
        ]
        self.enforcer.enforce.assert_has_calls(expected_calls)

    def test_get_member(self):
        self.policy.get_member()
        expected_calls = [
            mock.call(self.context, 'get_image', mock.ANY),
            mock.call(self.context, 'get_member', mock.ANY)
        ]
        self.enforcer.enforce.assert_has_calls(expected_calls)

    def test_get_members(self):
        self.policy.get_members()
        expected_calls = [
            mock.call(self.context, 'get_image', mock.ANY),
            mock.call(self.context, 'get_members', mock.ANY)
        ]
        self.enforcer.enforce.assert_has_calls(expected_calls)

    def test_add_member(self):
        self.policy.add_member()
        expected_calls = [
            mock.call(self.context, 'get_image', mock.ANY),
            mock.call(self.context, 'add_member', mock.ANY)
        ]
        self.enforcer.enforce.assert_has_calls(expected_calls)

    def test_modify_member(self):
        self.policy.modify_member()
        expected_calls = [
            mock.call(self.context, 'get_image', mock.ANY),
            mock.call(self.context, 'modify_member', mock.ANY)
        ]
        self.enforcer.enforce.assert_has_calls(expected_calls)

    def test_delete_member(self):
        self.policy.delete_member()
        expected_calls = [
            mock.call(self.context, 'get_image', mock.ANY),
            mock.call(self.context, 'delete_member', mock.ANY)
        ]
        self.enforcer.enforce.assert_has_calls(expected_calls)

    def test_enforce_exception_behavior(self):
        with mock.patch.object(self.policy.enforcer, 'enforce') as mock_enf:
            # First make sure we can update if allowed
            self.policy.modify_member()
            self.assertTrue(mock_enf.called)

            # Make sure that if while checking modify_member if get_image
            # both returns forbidden then we should get NotFound. This is
            # because we are not allowed to fetch image details.
            mock_enf.reset_mock()
            mock_enf.side_effect = exception.Forbidden
            self.assertRaises(webob.exc.HTTPNotFound,
                              self.policy.modify_member)
            # Make sure we checked modify_image, and then get_image.
            mock_enf.assert_has_calls([
                mock.call(mock.ANY, 'get_image', mock.ANY)])

            # Make sure that if modify_member is disallowed, but
            # get_image is allowed, that we get Forbidden. This is
            # because we are allowed to see the image, but not modify
            # it, so 403 indicates that without confusing the user and
            # returning "not found" for an image they are able to GET.
            mock_enf.reset_mock()
            mock_enf.side_effect = [lambda *a: None, exception.Forbidden]
            self.assertRaises(webob.exc.HTTPForbidden,
                              self.policy.modify_member)
            # Make sure we checked get_image, and then modify_member.
            mock_enf.assert_has_calls([
                mock.call(mock.ANY, 'get_image', mock.ANY),
                mock.call(mock.ANY, 'modify_member', mock.ANY)])


class TestTasksAPIPolicy(APIPolicyBase):
    def setUp(self):
        super(TestTasksAPIPolicy, self).setUp()
        self.enforcer = mock.MagicMock()
        self.context = mock.MagicMock()
        self.policy = policy.TasksAPIPolicy(self.context,
                                            enforcer=self.enforcer)

    def test_tasks_api_access(self):
        self.policy.tasks_api_access()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'tasks_api_access',
                                                      mock.ANY)


class TestCacheImageAPIPolicy(utils.BaseTestCase):
    def setUp(self):
        super(TestCacheImageAPIPolicy, self).setUp()
        self.enforcer = mock.MagicMock()
        self.context = mock.MagicMock()

    def test_manage_image_cache(self):
        self.policy = policy.CacheImageAPIPolicy(
            self.context, enforcer=self.enforcer,
            policy_str='manage_image_cache')
        self.policy.manage_image_cache()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'manage_image_cache',
                                                      mock.ANY)

    def test_manage_image_cache_with_cache_delete(self):
        self.policy = policy.CacheImageAPIPolicy(
            self.context, enforcer=self.enforcer,
            policy_str='cache_delete')
        self.policy.manage_image_cache()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'cache_delete',
                                                      mock.ANY)

    def test_manage_image_cache_with_cache_list(self):
        self.policy = policy.CacheImageAPIPolicy(
            self.context, enforcer=self.enforcer,
            policy_str='cache_list')
        self.policy.manage_image_cache()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'cache_list',
                                                      mock.ANY)

    def test_manage_image_cache_with_cache_image(self):
        self.policy = policy.CacheImageAPIPolicy(
            self.context, enforcer=self.enforcer,
            policy_str='cache_image')
        self.policy.manage_image_cache()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'cache_image',
                                                      mock.ANY)


class TestDiscoveryAPIPolicy(APIPolicyBase):
    def setUp(self):
        super(TestDiscoveryAPIPolicy, self).setUp()
        self.enforcer = mock.MagicMock()
        self.context = mock.MagicMock()
        self.policy = policy.DiscoveryAPIPolicy(
            self.context, enforcer=self.enforcer)

    def test_stores_info_detail(self):
        self.policy.stores_info_detail()
        self.enforcer.enforce.assert_called_once_with(self.context,
                                                      'stores_info_detail',
                                                      mock.ANY)



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime
import http.client as http

import glance_store
from oslo_config import cfg
from oslo_serialization import jsonutils
import webob

import glance.api.v2.image_members
import glance.tests.unit.utils as unit_test_utils
import glance.tests.utils as test_utils

DATETIME = datetime.datetime(2012, 5, 16, 15, 27, 36, 325355)
ISOTIME = '2012-05-16T15:27:36Z'


CONF = cfg.CONF

BASE_URI = unit_test_utils.BASE_URI


UUID1 = 'c80a1a6c-bd1f-41c5-90ee-81afedb1d58d'
UUID2 = 'a85abd86-55b3-4d5b-b0b4-5d0a6e6042fc'
UUID3 = '971ec09a-8067-4bc8-a91f-ae3557f1c4c7'
UUID4 = '6bbe7cc2-eae7-4c0f-b50d-a7160b0c6a86'
UUID5 = '3eee7cc2-eae7-4c0f-b50d-a7160b0c62ed'

TENANT1 = '6838eb7b-6ded-434a-882c-b344c77fe8df'
TENANT2 = '2c014f32-55eb-467d-8fcb-4bd706012f81'
TENANT3 = '5a3e60e8-cfa9-4a9e-a90a-62b42cea92b8'
TENANT4 = 'c6c87f25-8a94-47ed-8c83-053c25f42df4'


def _db_fixture(id, **kwargs):
    obj = {
        'id': id,
        'name': None,
        'visibility': 'shared',
        'properties': {},
        'checksum': None,
        'owner': None,
        'status': 'queued',
        'tags': [],
        'size': None,
        'locations': [],
        'protected': False,
        'disk_format': None,
        'container_format': None,
        'deleted': False,
        'min_ram': None,
        'min_disk': None,
    }
    obj.update(kwargs)
    return obj


def _db_image_member_fixture(image_id, member_id, **kwargs):
    obj = {
        'image_id': image_id,
        'member': member_id,
        'status': 'pending',
    }
    obj.update(kwargs)
    return obj


def _domain_fixture(id, **kwargs):
    properties = {
        'id': id,
    }
    properties.update(kwargs)
    return glance.domain.ImageMembership(**properties)


class TestImageMembersController(test_utils.BaseTestCase):

    def setUp(self):
        super(TestImageMembersController, self).setUp()
        self.db = unit_test_utils.FakeDB(initialize=False)
        self.store = unit_test_utils.FakeStoreAPI()
        self.policy = unit_test_utils.FakePolicyEnforcer()
        self.notifier = unit_test_utils.FakeNotifier()
        self._create_images()
        self._create_image_members()
        self.controller = glance.api.v2.image_members.ImageMembersController(
            self.db,
            self.policy,
            self.notifier,
            self.store)
        glance_store.register_opts(CONF)

        self.config(default_store='filesystem',
                    filesystem_store_datadir=self.test_dir,
                    group="glance_store")

        glance_store.create_stores()

    def _create_images(self):
        self.images = [
            _db_fixture(UUID1, owner=TENANT1, name='1', size=256,
                        visibility='public',
                        locations=[{'url': '%s/%s' % (BASE_URI, UUID1),
                                    'metadata': {}, 'status': 'active'}]),
            _db_fixture(UUID2, owner=TENANT1, name='2', size=512),
            _db_fixture(UUID3, owner=TENANT3, name='3', size=512),
            _db_fixture(UUID4, owner=TENANT4, name='4', size=1024),
            _db_fixture(UUID5, owner=TENANT1, name='5', size=1024),
        ]
        [self.db.image_create(None, image) for image in self.images]

        self.db.image_tag_set_all(None, UUID1, ['ping', 'pong'])

    def _create_image_members(self):
        self.image_members = [
            _db_image_member_fixture(UUID2, TENANT4),
            _db_image_member_fixture(UUID3, TENANT4),
            _db_image_member_fixture(UUID3, TENANT2),
            _db_image_member_fixture(UUID4, TENANT1),
        ]
        [self.db.image_member_create(None, image_member)
            for image_member in self.image_members]

    def test_index(self):
        request = unit_test_utils.get_fake_request()
        output = self.controller.index(request, UUID2)
        self.assertEqual(1, len(output['members']))
        actual = set([image_member.member_id
                      for image_member in output['members']])
        expected = set([TENANT4])
        self.assertEqual(expected, actual)

    def test_index_no_members(self):
        request = unit_test_utils.get_fake_request()
        output = self.controller.index(request, UUID5)
        self.assertEqual(0, len(output['members']))
        self.assertEqual({'members': []}, output)

    def test_index_member_view(self):
        # UUID3 is a shared image owned by TENANT3
        # UUID3 has members TENANT2 and TENANT4
        # When TENANT4 lists members for UUID3, should not see TENANT2
        request = unit_test_utils.get_fake_request(tenant=TENANT4)
        output = self.controller.index(request, UUID3)
        self.assertEqual(1, len(output['members']))
        actual = set([image_member.member_id
                      for image_member in output['members']])
        expected = set([TENANT4])
        self.assertEqual(expected, actual)

    def test_index_private_image(self):
        request = unit_test_utils.get_fake_request(tenant=TENANT2)
        self.assertRaises(webob.exc.HTTPNotFound, self.controller.index,
                          request, UUID5)

    def test_index_public_image(self):
        request = unit_test_utils.get_fake_request()
        self.assertRaises(webob.exc.HTTPForbidden, self.controller.index,
                          request, UUID1)

    def test_index_private_image_visible_members_admin(self):
        request = unit_test_utils.get_fake_request(is_admin=True)
        output = self.controller.index(request, UUID4)
        self.assertEqual(1, len(output['members']))
        actual = set([image_member.member_id
                      for image_member in output['members']])
        expected = set([TENANT1])
        self.assertEqual(expected, actual)

    def test_index_allowed_by_get_members_policy(self):
        rules = {"get_members": True}
        self.policy.set_rules(rules)
        request = unit_test_utils.get_fake_request()
        output = self.controller.index(request, UUID2)
        self.assertEqual(1, len(output['members']))

    def test_index_forbidden_by_get_members_policy(self):
        rules = {"get_members": False}
        self.policy.set_rules(rules)
        request = unit_test_utils.get_fake_request()
        self.assertRaises(webob.exc.HTTPForbidden, self.controller.index,
                          request, image_id=UUID2)

    def test_show(self):
        request = unit_test_utils.get_fake_request(tenant=TENANT1)
        output = self.controller.show(request, UUID2, TENANT4)
        expected = self.image_members[0]
        self.assertEqual(expected['image_id'], output.image_id)
        self.assertEqual(expected['member'], output.member_id)
        self.assertEqual(expected['status'], output.status)

    def test_show_by_member(self):
        request = unit_test_utils.get_fake_request(tenant=TENANT4)
        output = self.controller.show(request, UUID2, TENANT4)
        expected = self.image_members[0]
        self.assertEqual(expected['image_id'], output.image_id)
        self.assertEqual(expected['member'], output.member_id)
        self.assertEqual(expected['status'], output.status)

    def test_show_forbidden(self):
        request = unit_test_utils.get_fake_request(tenant=TENANT2)
        self.assertRaises(webob.exc.HTTPNotFound, self.controller.show,
                          request, UUID2, TENANT4)

    def test_show_not_found(self):
        # one member should not be able to view status of another member
        # of the same image
        request = unit_test_utils.get_fake_request(tenant=TENANT2)
        self.assertRaises(webob.exc.HTTPNotFound, self.controller.show,
                          request, UUID3, TENANT4)

    def test_create(self):
        request = unit_test_utils.get_fake_request()
        image_id = UUID2
        member_id = TENANT3
        output = self.controller.create(request, image_id=image_id,
                                        member_id=member_id)
        self.assertEqual(UUID2, output.image_id)
        self.assertEqual(TENANT3, output.member_id)

    def test_create_allowed_by_add_policy(self):
        rules = {"add_member": True}
        self.policy.set_rules(rules)
        request = unit_test_utils.get_fake_request()
        output = self.controller.create(request, image_id=UUID2,
                                        member_id=TENANT3)
        self.assertEqual(UUID2, output.image_id)
        self.assertEqual(TENANT3, output.member_id)

    def test_create_forbidden_by_add_policy(self):
        rules = {"add_member": False}
        self.policy.set_rules(rules)
        request = unit_test_utils.get_fake_request()
        self.assertRaises(webob.exc.HTTPForbidden, self.controller.create,
                          request, image_id=UUID2, member_id=TENANT3)

    def test_create_duplicate_member(self):
        request = unit_test_utils.get_fake_request()
        image_id = UUID2
        member_id = TENANT3
        output = self.controller.create(request, image_id=image_id,
                                        member_id=member_id)
        self.assertEqual(UUID2, output.image_id)
        self.assertEqual(TENANT3, output.member_id)

        self.assertRaises(webob.exc.HTTPConflict, self.controller.create,
                          request, image_id=image_id, member_id=member_id)

    def test_create_overlimit(self):
        self.config(image_member_quota=0)
        request = unit_test_utils.get_fake_request()
        image_id = UUID2
        member_id = TENANT3
        self.assertRaises(webob.exc.HTTPRequestEntityTooLarge,
                          self.controller.create, request,
                          image_id=image_id, member_id=member_id)

    def test_create_unlimited(self):
        self.config(image_member_quota=-1)
        request = unit_test_utils.get_fake_request()
        image_id = UUID2
        member_id = TENANT3
        output = self.controller.create(request, image_id=image_id,
                                        member_id=member_id)
        self.assertEqual(UUID2, output.image_id)
        self.assertEqual(TENANT3, output.member_id)

    def test_member_create_raises_bad_request_for_unicode_value(self):
        request = unit_test_utils.get_fake_request()
        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
                          request, image_id=UUID5,
                          member_id='\U0001f693')

    def test_update_done_by_member(self):
        request = unit_test_utils.get_fake_request(tenant=TENANT4)
        image_id = UUID2
        member_id = TENANT4
        output = self.controller.update(request, image_id=image_id,
                                        member_id=member_id,
                                        status='accepted')
        self.assertEqual(UUID2, output.image_id)
        self.assertEqual(TENANT4, output.member_id)
        self.assertEqual('accepted', output.status)

    def test_update_done_by_member_forbidden_by_policy(self):
        rules = {"modify_member": False}
        self.policy.set_rules(rules)
        request = unit_test_utils.get_fake_request(tenant=TENANT4)
        self.assertRaises(webob.exc.HTTPForbidden, self.controller.update,
                          request, image_id=UUID2, member_id=TENANT4,
                          status='accepted')

    def test_update_done_by_member_allowed_by_policy(self):
        rules = {"modify_member": True}
        self.policy.set_rules(rules)
        request = unit_test_utils.get_fake_request(tenant=TENANT4)
        output = self.controller.update(request, image_id=UUID2,
                                        member_id=TENANT4,
                                        status='accepted')
        self.assertEqual(UUID2, output.image_id)
        self.assertEqual(TENANT4, output.member_id)
        self.assertEqual('accepted', output.status)

    def test_update_done_by_owner(self):
        enforcer = unit_test_utils.enforcer_from_rules({
            "get_image": "",
            "modify_image": "'{0}':%(owner)s".format(TENANT1)
        })
        self.controller.policy = enforcer
        request = unit_test_utils.get_fake_request(tenant=TENANT1)
        self.assertRaises(webob.exc.HTTPForbidden, self.controller.update,
                          request, UUID2, TENANT4, status='accepted')

    def test_update_non_existent_image(self):
        request = unit_test_utils.get_fake_request(tenant=TENANT1)
        self.assertRaises(webob.exc.HTTPNotFound, self.controller.update,
                          request, '123', TENANT4, status='accepted')

    def test_update_invalid_status(self):
        request = unit_test_utils.get_fake_request(tenant=TENANT4)
        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.update,
                          request, UUID2, TENANT4, status='accept')

    def test_create_private_image(self):
        enforcer = unit_test_utils.enforcer_from_rules({
            "get_image": "",
        })
        self.controller.policy = enforcer
        request = unit_test_utils.get_fake_request()
        self.assertRaises(webob.exc.HTTPForbidden, self.controller.create,
                          request, UUID4, TENANT2)

    def test_create_public_image(self):
        request = unit_test_utils.get_fake_request()
        self.assertRaises(webob.exc.HTTPForbidden, self.controller.create,
                          request, UUID1, TENANT2)

    def test_create_image_does_not_exist(self):
        request = unit_test_utils.get_fake_request()
        image_id = 'fake-image-id'
        member_id = TENANT3
        self.assertRaises(webob.exc.HTTPNotFound, self.controller.create,
                          request, image_id=image_id, member_id=member_id)

    def test_delete(self):
        request = unit_test_utils.get_fake_request()
        member_id = TENANT4
        image_id = UUID2
        res = self.controller.delete(request, image_id, member_id)
        self.assertEqual(b'', res.body)
        self.assertEqual(http.NO_CONTENT, res.status_code)
        found_member = self.db.image_member_find(
            request.context, image_id=image_id, member=member_id)
        self.assertEqual([], found_member)

    def test_delete_by_member(self):
        enforcer = unit_test_utils.enforcer_from_rules({
            "get_image": "",
            "delete_member": "'{0}':%(owner)s".format(TENANT4),
            "get_members": "",
            "get_member": ""
        })
        request = unit_test_utils.get_fake_request(tenant=TENANT4)
        self.controller.policy = enforcer
        self.assertRaises(webob.exc.HTTPForbidden, self.controller.delete,
                          request, UUID2, TENANT4)
        request = unit_test_utils.get_fake_request()
        output = self.controller.index(request, UUID2)
        self.assertEqual(1, len(output['members']))
        actual = set([image_member.member_id
                      for image_member in output['members']])
        expected = set([TENANT4])
        self.assertEqual(expected, actual)

    def test_delete_allowed_by_policies(self):
        rules = {"get_member": True, "delete_member": True}
        self.policy.set_rules(rules)
        request = unit_test_utils.get_fake_request(tenant=TENANT1)
        output = self.controller.delete(request, image_id=UUID2,
                                        member_id=TENANT4)
        request = unit_test_utils.get_fake_request()
        output = self.controller.index(request, UUID2)
        self.assertEqual(0, len(output['members']))

    def test_delete_forbidden_by_get_member_policy(self):
        rules = {"get_member": False}
        self.policy.set_rules(rules)
        request = unit_test_utils.get_fake_request(tenant=TENANT1)
        self.assertRaises(webob.exc.HTTPForbidden, self.controller.delete,
                          request, UUID2, TENANT4)

    def test_delete_forbidden_by_delete_member_policy(self):
        rules = {"delete_member": False}
        self.policy.set_rules(rules)
        request = unit_test_utils.get_fake_request(tenant=TENANT1)
        self.assertRaises(webob.exc.HTTPForbidden, self.controller.delete,
                          request, UUID2, TENANT4)

    def test_delete_private_image(self):
        enforcer = unit_test_utils.enforcer_from_rules({
            "get_image": "",
            "delete_member": "'{0}':%(owner)s".format(TENANT1),
            "get_member": ""
        })
        self.controller.policy = enforcer
        request = unit_test_utils.get_fake_request()
        self.assertRaises(webob.exc.HTTPForbidden, self.controller.delete,
                          request, UUID4, TENANT1)

    def test_delete_public_image(self):
        request = unit_test_utils.get_fake_request()
        self.assertRaises(webob.exc.HTTPForbidden, self.controller.delete,
                          request, UUID1, TENANT1)

    def test_delete_image_does_not_exist(self):
        request = unit_test_utils.get_fake_request()
        member_id = TENANT2
        image_id = 'fake-image-id'
        self.assertRaises(webob.exc.HTTPNotFound, self.controller.delete,
                          request, image_id, member_id)

    def test_delete_member_does_not_exist(self):
        request = unit_test_utils.get_fake_request()
        member_id = 'fake-member-id'
        image_id = UUID2
        found_member = self.db.image_member_find(
            request.context, image_id=image_id, member=member_id)
        self.assertEqual([], found_member)
        self.assertRaises(webob.exc.HTTPNotFound, self.controller.delete,
                          request, image_id, member_id)


class TestImageMembersSerializer(test_utils.BaseTestCase):

    def setUp(self):
        super(TestImageMembersSerializer, self).setUp()
        self.serializer = glance.api.v2.image_members.ResponseSerializer()
        self.fixtures = [
            _domain_fixture(id='1', image_id=UUID2, member_id=TENANT1,
                            status='accepted',
                            created_at=DATETIME, updated_at=DATETIME),
            _domain_fixture(id='2', image_id=UUID2, member_id=TENANT2,
                            status='pending',
                            created_at=DATETIME, updated_at=DATETIME),
        ]

    def test_index(self):
        expected = {
            'members': [
                {
                    'image_id': UUID2,
                    'member_id': TENANT1,
                    'status': 'accepted',
                    'created_at': ISOTIME,
                    'updated_at': ISOTIME,
                    'schema': '/v2/schemas/member',
                },
                {
                    'image_id': UUID2,
                    'member_id': TENANT2,
                    'status': 'pending',
                    'created_at': ISOTIME,
                    'updated_at': ISOTIME,
                    'schema': '/v2/schemas/member',
                },
            ],
            'schema': '/v2/schemas/members',
        }
        request = webob.Request.blank('/v2/images/%s/members' % UUID2)
        response = webob.Response(request=request)
        result = {'members': self.fixtures}
        self.serializer.index(response, result)
        actual = jsonutils.loads(response.body)
        self.assertEqual(expected, actual)
        self.assertEqual('application/json', response.content_type)

    def test_show(self):
        expected = {
            'image_id': UUID2,
            'member_id': TENANT1,
            'status': 'accepted',
            'created_at': ISOTIME,
            'updated_at': ISOTIME,
            'schema': '/v2/schemas/member',
        }
        request = webob.Request.blank('/v2/images/%s/members/%s'
                                      % (UUID2, TENANT1))
        response = webob.Response(request=request)
        result = self.fixtures[0]
        self.serializer.show(response, result)
        actual = jsonutils.loads(response.body)
        self.assertEqual(expected, actual)
        self.assertEqual('application/json', response.content_type)

    def test_create(self):
        expected = {'image_id': UUID2,
                    'member_id': TENANT1,
                    'status': 'accepted',
                    'schema': '/v2/schemas/member',
                    'created_at': ISOTIME,
                    'updated_at': ISOTIME}
        request = webob.Request.blank('/v2/images/%s/members/%s'
                                      % (UUID2, TENANT1))
        response = webob.Response(request=request)
        result = self.fixtures[0]
        self.serializer.create(response, result)
        actual = jsonutils.loads(response.body)
        self.assertEqual(expected, actual)
        self.assertEqual('application/json', response.content_type)

    def test_update(self):
        expected = {'image_id': UUID2,
                    'member_id': TENANT1,
                    'status': 'accepted',
                    'schema': '/v2/schemas/member',
                    'created_at': ISOTIME,
                    'updated_at': ISOTIME}
        request = webob.Request.blank('/v2/images/%s/members/%s'
                                      % (UUID2, TENANT1))
        response = webob.Response(request=request)
        result = self.fixtures[0]
        self.serializer.update(response, result)
        actual = jsonutils.loads(response.body)
        self.assertEqual(expected, actual)
        self.assertEqual('application/json', response.content_type)


class TestImagesDeserializer(test_utils.BaseTestCase):

    def setUp(self):
        super(TestImagesDeserializer, self).setUp()
        self.deserializer = glance.api.v2.image_members.RequestDeserializer()

    def test_create(self):
        request = unit_test_utils.get_fake_request()
        request.body = jsonutils.dump_as_bytes({'member': TENANT1})
        output = self.deserializer.create(request)
        expected = {'member_id': TENANT1}
        self.assertEqual(expected, output)

    def test_create_invalid(self):
        request = unit_test_utils.get_fake_request()
        request.body = jsonutils.dump_as_bytes({'mem': TENANT1})
        self.assertRaises(webob.exc.HTTPBadRequest, self.deserializer.create,
                          request)

    def test_create_no_body(self):
        request = unit_test_utils.get_fake_request()
        self.assertRaises(webob.exc.HTTPBadRequest, self.deserializer.create,
                          request)

    def test_create_member_empty(self):
        request = unit_test_utils.get_fake_request()
        request.body = jsonutils.dump_as_bytes({'member': ''})
        self.assertRaises(webob.exc.HTTPBadRequest, self.deserializer.create,
                          request)

    def test_create_list_return_error(self):
        request = unit_test_utils.get_fake_request()
        request.body = jsonutils.dump_as_bytes([TENANT1])
        self.assertRaises(webob.exc.HTTPBadRequest, self.deserializer.create,
                          request)

    def test_update_list_return_error(self):
        request = unit_test_utils.get_fake_request()
        request.body = jsonutils.dump_as_bytes([TENANT1])
        self.assertRaises(webob.exc.HTTPBadRequest, self.deserializer.update,
                          request)

    def test_update(self):
        request = unit_test_utils.get_fake_request()
        request.body = jsonutils.dump_as_bytes({'status': 'accepted'})
        output = self.deserializer.update(request)
        expected = {'status': 'accepted'}
        self.assertEqual(expected, output)

    def test_update_invalid(self):
        request = unit_test_utils.get_fake_request()
        request.body = jsonutils.dump_as_bytes({'mem': TENANT1})
        self.assertRaises(webob.exc.HTTPBadRequest, self.deserializer.update,
                          request)

    def test_update_no_body(self):
        request = unit_test_utils.get_fake_request()
        self.assertRaises(webob.exc.HTTPBadRequest, self.deserializer.update,
                          request)



# Copyright 2011 OpenStack Foundation
# Copyright 2012 Red Hat, Inc
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Functional test asserting strongly typed exceptions from glance client"""

import http.client as http_client

import eventlet.patcher
import httplib2
import webob.dec
import webob.exc

from glance.common import client
from glance.common import exception
from glance.common import wsgi
from glance.tests import utils

eventlet.patcher.monkey_patch(socket=True)


class ExceptionTestApp(object):
    """
    Test WSGI application which can respond with multiple kinds of HTTP
    status codes
    """

    @webob.dec.wsgify
    def __call__(self, request):
        path = request.path_qs

        if path == "/rate-limit":
            request.response = webob.exc.HTTPRequestEntityTooLarge()

        elif path == "/rate-limit-retry":
            request.response.retry_after = 10
            request.response.status = http_client.REQUEST_ENTITY_TOO_LARGE

        elif path == "/service-unavailable":
            request.response = webob.exc.HTTPServiceUnavailable()

        elif path == "/service-unavailable-retry":
            request.response.retry_after = 10
            request.response.status = http_client.SERVICE_UNAVAILABLE

        elif path == "/expectation-failed":
            request.response = webob.exc.HTTPExpectationFailed()

        elif path == "/server-error":
            request.response = webob.exc.HTTPServerError()

        elif path == "/server-traceback":
            raise exception.ServerError()


class TestClientExceptions(utils.BaseTestCase):

    def setUp(self):
        super(TestClientExceptions, self).setUp()
        self.port = utils.get_unused_port()
        server = wsgi.Server()
        self.config(bind_host='127.0.0.1')
        self.config(workers=0)
        server.start(ExceptionTestApp(), self.port)
        self.client = client.BaseClient("127.0.0.1", self.port)

    def _do_test_exception(self, path, exc_type):
        try:
            self.client.do_request("GET", path)
            self.fail('expected %s' % exc_type)
        except exc_type as e:
            if 'retry' in path:
                self.assertEqual(10, e.retry_after)

    def test_rate_limited(self):
        """
        Test rate limited response
        """
        self._do_test_exception('/rate-limit', exception.LimitExceeded)

    def test_rate_limited_retry(self):
        """
        Test rate limited response with retry
        """
        self._do_test_exception('/rate-limit-retry', exception.LimitExceeded)

    def test_service_unavailable(self):
        """
        Test service unavailable response
        """
        self._do_test_exception('/service-unavailable',
                                exception.ServiceUnavailable)

    def test_service_unavailable_retry(self):
        """
        Test service unavailable response with retry
        """
        self._do_test_exception('/service-unavailable-retry',
                                exception.ServiceUnavailable)

    def test_expectation_failed(self):
        """
        Test expectation failed response
        """
        self._do_test_exception('/expectation-failed',
                                exception.UnexpectedStatus)

    def test_server_error(self):
        """
        Test server error response
        """
        self._do_test_exception('/server-error',
                                exception.ServerError)

    def test_server_traceback(self):
        """
        Verify that the wsgi server does not return tracebacks to the client on
        500 errors (bug 1192132)
        """
        http = httplib2.Http()
        path = ('http://%s:%d/server-traceback' %
                ('127.0.0.1', self.port))
        response, content = http.request(path, 'GET')
        self.assertNotIn(b'ServerError', content)
        self.assertEqual(http_client.INTERNAL_SERVER_ERROR, response.status)



# Copyright (c) 2017 RedHat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import glance.api.v2.discovery
import glance.tests.unit.utils as unit_test_utils
import glance.tests.utils as test_utils


class TestInfoControllers(test_utils.BaseTestCase):

    def setUp(self):
        super(TestInfoControllers, self).setUp()
        self.controller = glance.api.v2.discovery.InfoController()

    def test_get_import_info_with_empty_method_list(self):
        """When methods list is empty, should still return import methods"""
        self.config(enabled_import_methods=[])
        req = unit_test_utils.get_fake_request()
        output = self.controller.get_image_import(req)
        self.assertIn('import-methods', output)
        self.assertEqual([], output['import-methods']['value'])

    def test_get_import_info(self):
        """Testing defaults, not all possible values"""
        default_import_methods = ['glance-direct', 'web-download',
                                  'copy-image']

        req = unit_test_utils.get_fake_request()
        output = self.controller.get_image_import(req)
        self.assertIn('import-methods', output)
        self.assertEqual(default_import_methods,
                         output['import-methods']['value'])



# Copyright 2021 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import fixtures
import http.client as http

from oslo_utils import units

from glance.quota import keystone as ks_quota
from glance.tests import functional
from glance.tests.functional.v2.test_images import get_enforcer_class
from glance.tests import utils as test_utils


class TestDiscovery(functional.SynchronousAPIBase):
    def setUp(self):
        super(TestDiscovery, self).setUp()
        self.config(endpoint_id='ENDPOINT_ID', group='oslo_limit')
        self.config(use_keystone_limits=True)

        self.enforcer_mock = self.useFixture(
            fixtures.MockPatchObject(ks_quota, 'limit')).mock

    def set_limit(self, limits):
        self.enforcer_mock.Enforcer = get_enforcer_class(limits)

    def _assert_usage(self, expected):
        usage = self.api_get('/v2/info/usage')
        usage = usage.json['usage']
        for item in ('count', 'size', 'stage'):
            key = 'image_%s_total' % item
            self.assertEqual(expected[key], usage[key],
                             'Mismatch in %s' % key)
        self.assertEqual(expected['image_count_uploading'],
                         usage['image_count_uploading'])

    def test_quota_with_usage(self):
        self.set_limit({'image_size_total': 5,
                        'image_count_total': 10,
                        'image_stage_total': 15,
                        'image_count_uploading': 20})

        self.start_server()

        # Initially we expect no usage, but our limits in place.
        expected = {
            'image_size_total': {'limit': 5, 'usage': 0},
            'image_count_total': {'limit': 10, 'usage': 0},
            'image_stage_total': {'limit': 15, 'usage': 0},
            'image_count_uploading': {'limit': 20, 'usage': 0},
        }
        self._assert_usage(expected)

        # Stage 1MiB and see our total count, uploading count, and
        # staging area usage increase.
        data = test_utils.FakeData(1 * units.Mi)
        image_id = self._create_and_stage(data_iter=data)
        expected['image_count_uploading']['usage'] = 1
        expected['image_count_total']['usage'] = 1
        expected['image_stage_total']['usage'] = 1
        self._assert_usage(expected)

        # Doing the import does not change anything (since we are
        # synchronous and the task will not have run yet).
        self._import_direct(image_id, ['store1'])
        self._assert_usage(expected)

        # After the import is complete, our usage of the staging area
        # drops to zero, and our consumption of actual store space
        # reflects the new active image.
        self._wait_for_import(image_id)
        expected['image_count_uploading']['usage'] = 0
        expected['image_stage_total']['usage'] = 0
        expected['image_size_total']['usage'] = 1
        self._assert_usage(expected)

        # Upload also yields a new active image and store usage.
        data = test_utils.FakeData(1 * units.Mi)
        image_id = self._create_and_upload(data_iter=data)
        expected['image_count_total']['usage'] = 2
        expected['image_size_total']['usage'] = 2
        self._assert_usage(expected)

        # Deleting an image drops the usage down.
        self.api_delete('/v2/images/%s' % image_id)
        expected['image_count_total']['usage'] = 1
        expected['image_size_total']['usage'] = 1
        self._assert_usage(expected)

    def test_stores(self):
        # NOTE(mrjoshi): As this is a functional test, we are
        # testing the functionality with file stores.

        self.start_server()

        # If user is admin or non-admin the store list will be
        # displayed.
        stores = self.api_get('/v2/info/stores').json['stores']
        expected = {
            "stores": [
                {
                    "id": "store1",
                    "default": "true"
                },
                {
                    "id": "store2"
                },
                {
                    "id": "store3"
                }]}

        self.assertEqual(expected['stores'], stores)

        # If user is admin the store list will be displayed
        # along with store properties.
        stores = self.api_get('/v2/info/stores/detail').json['stores']
        expected = {
            "stores": [
                {
                    "id": "store1",
                    "default": "true",
                    "type": "file",
                    "weight": 0,
                    "properties": {
                          "data_dir": self._store_dir('store1'),
                          "chunk_size": 65536,
                          "thin_provisioning": False
                    }
                },
                {
                    "id": "store2",
                    "type": "file",
                    "weight": 0,
                    "properties": {
                          "data_dir": self._store_dir('store2'),
                          "chunk_size": 65536,
                          "thin_provisioning": False
                    }
                },
                {
                    "id": "store3",
                    "type": "file",
                    "weight": 0,
                    "properties": {
                          "data_dir": self._store_dir('store3'),
                          "chunk_size": 65536,
                          "thin_provisioning": False
                    }
                }]}

        self.assertEqual(expected['stores'], stores)

        # If user is non-admin 403 Error response will be returned.
        response = self.api_get('/v2/info/stores/detail',
                                headers={'X-Roles': 'member'})
        self.assertEqual(http.FORBIDDEN, response.status_code)



# Copyright 2015 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

from oslo_concurrency import processutils as putils
from oslo_config import cfg
from oslo_log import log as logging
from taskflow.patterns import linear_flow as lf
from taskflow import task

from glance.i18n import _, _LW

LOG = logging.getLogger(__name__)

convert_task_opts = [
    # NOTE: This configuration option requires the operator to explicitly set
    # an image conversion format. There being no sane default due to the
    # dependency on the environment in which OpenStack is running, we do not
    # mark this configuration option as "required". Rather a warning message
    # is given to the operator, prompting for an image conversion format to
    # be set.
    cfg.StrOpt('conversion_format',
               sample_default='raw',
               choices=('qcow2', 'raw', 'vmdk'),
               help=_("""
Set the desired image conversion format.

Provide a valid image format to which you want images to be
converted before they are stored for consumption by Glance.
Appropriate image format conversions are desirable for specific
storage backends in order to facilitate efficient handling of
bandwidth and usage of the storage infrastructure.

By default, ``conversion_format`` is not set and must be set
explicitly in the configuration file.

The allowed values for this option are ``raw``, ``qcow2`` and
``vmdk``. The  ``raw`` format is the unstructured disk format and
should be chosen when RBD or Ceph storage backends are used for
image storage. ``qcow2`` is supported by the QEMU emulator that
expands dynamically and supports Copy on Write. The ``vmdk`` is
another common disk format supported by many common virtual machine
monitors like VMWare Workstation.

Possible values:
    * qcow2
    * raw
    * vmdk

Related options:
    * disk_formats

""")),
]

CONF = cfg.CONF

# NOTE(flaper87): Registering under the taskflow_executor section
# for now. It seems a waste to have a whole section dedicated to a
# single task with a single option.
CONF.register_opts(convert_task_opts, group='taskflow_executor')


class _Convert(task.Task):

    conversion_missing_warned = False

    def __init__(self, task_id, task_type, image_repo):
        self.task_id = task_id
        self.task_type = task_type
        self.image_repo = image_repo
        super(_Convert, self).__init__(
            name='%s-Convert-%s' % (task_type, task_id))

    def execute(self, image_id, file_path):

        # NOTE(flaper87): A format must be explicitly
        # specified. There's no "sane" default for this
        # because the dest format may work differently depending
        # on the environment OpenStack is running in.
        abs_file_path = file_path.split("file://")[-1]
        conversion_format = CONF.taskflow_executor.conversion_format
        if conversion_format is None:
            if not _Convert.conversion_missing_warned:
                msg = _LW('The conversion format is None, please add a value '
                          'for it in the config file for this task to '
                          'work: %s')
                LOG.warning(msg, self.task_id)
                _Convert.conversion_missing_warned = True
            return

        image_obj = self.image_repo.get(image_id)
        src_format = image_obj.disk_format

        # TODO(flaper87): Check whether the image is in the desired
        # format already. Probably using `qemu-img` just like the
        # `Introspection` task.

        # NOTE(hemanthm): We add '-f' parameter to the convert command here so
        # that the image format need not be inferred by qemu utils. This
        # shields us from being vulnerable to an attack vector described here
        # https://bugs.launchpad.net/glance/+bug/1449062

        data_dir = CONF.task.work_dir
        # NOTE(abhishekk): Use reserved 'os_glance_tasks_store' for tasks.
        if CONF.enabled_backends:
            data_dir = getattr(
                CONF, 'os_glance_tasks_store').filesystem_store_datadir

        dest_path = os.path.join(data_dir, "%s.converted" % image_id)
        stdout, stderr = putils.trycmd('qemu-img', 'convert',
                                       '-f', src_format,
                                       '-O', conversion_format,
                                       file_path, dest_path,
                                       log_errors=putils.LOG_ALL_ERRORS)

        if stderr:
            raise RuntimeError(stderr)

        os.unlink(abs_file_path)
        os.rename(dest_path, abs_file_path)
        return file_path

    def revert(self, image_id, result=None, **kwargs):
        # NOTE(flaper87): If result is None, it probably
        # means this task failed. Otherwise, we would have
        # a result from its execution.
        if result is None:
            return

        fs_path = result.split("file://")[-1]
        if os.path.exists(fs_path):
            os.remove(fs_path)


def get_flow(**kwargs):
    """Return task flow for converting images to different formats.

    :param task_id: Task ID.
    :param task_type: Type of the task.
    :param image_repo: Image repository used.
    """
    task_id = kwargs.get('task_id')
    task_type = kwargs.get('task_type')
    image_repo = kwargs.get('image_repo')

    return lf.Flow(task_type).add(
        _Convert(task_id, task_type, image_repo),
    )



# Copyright 2015 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

import futurist
import glance_store
from oslo_config import cfg
from taskflow import engines

import glance.async_
from glance.async_ import taskflow_executor
from glance.common.scripts.image_import import main as image_import
from glance import domain
import glance.tests.utils as test_utils

CONF = cfg.CONF
TENANT1 = '6838eb7b-6ded-434a-882c-b344c77fe8df'


class TestTaskExecutor(test_utils.BaseTestCase):

    def setUp(self):
        # NOTE(danms): Makes sure that we have a model set to something
        glance.async_._THREADPOOL_MODEL = None
        glance.async_.set_threadpool_model('eventlet')

        super(TestTaskExecutor, self).setUp()

        glance_store.register_opts(CONF)
        self.config(default_store='file',
                    stores=['file', 'http'],
                    filesystem_store_datadir=self.test_dir,
                    group="glance_store")
        glance_store.create_stores(CONF)

        self.config(engine_mode='serial',
                    group='taskflow_executor')

        self.context = mock.Mock()
        self.task_repo = mock.Mock()
        self.image_repo = mock.Mock()
        self.image_factory = mock.Mock()

        task_input = {
            "import_from": "http://cloud.foo/image.qcow2",
            "import_from_format": "qcow2",
            "image_properties": {'disk_format': 'qcow2',
                                 'container_format': 'bare'}
        }
        task_ttl = CONF.task.task_time_to_live

        self.task_type = 'import'
        image_id = 'fake-image-id'
        request_id = 'fake_request_id'
        user_id = 'fake_user'
        self.task_factory = domain.TaskFactory()
        self.task = self.task_factory.new_task(self.task_type, TENANT1,
                                               image_id, user_id,
                                               request_id,
                                               task_time_to_live=task_ttl,
                                               task_input=task_input)

        self.executor = taskflow_executor.TaskExecutor(
            self.context,
            self.task_repo,
            self.image_repo,
            self.image_factory)

    def test_fetch_an_executor_parallel(self):
        self.config(engine_mode='parallel', group='taskflow_executor')
        pool = self.executor._fetch_an_executor()
        self.assertIsInstance(pool, futurist.GreenThreadPoolExecutor)

    def test_fetch_an_executor_serial(self):
        pool = self.executor._fetch_an_executor()
        self.assertIsNone(pool)

    def test_begin_processing(self):
        with mock.patch.object(engines, 'load') as load_mock:
            engine = mock.Mock()
            load_mock.return_value = engine
            self.task_repo.get.return_value = self.task
            self.executor.begin_processing(self.task.task_id)

        # assert the call
        self.assertEqual(1, load_mock.call_count)
        self.assertEqual(1, engine.run.call_count)

    def test_task_fail(self):
        with mock.patch.object(engines, 'load') as load_mock:
            engine = mock.Mock()
            load_mock.return_value = engine
            engine.run.side_effect = RuntimeError
            self.task_repo.get.return_value = self.task
            self.assertRaises(RuntimeError, self.executor.begin_processing,
                              self.task.task_id)
        self.assertEqual('failure', self.task.status)
        self.task_repo.save.assert_called_with(self.task)

    def test_task_fail_upload(self):
        with mock.patch.object(image_import, 'set_image_data') as import_mock:
            import_mock.side_effect = IOError  # noqa

            self.task_repo.get.return_value = self.task
            self.executor.begin_processing(self.task.task_id)

        self.assertEqual('failure', self.task.status)
        self.task_repo.save.assert_called_with(self.task)
        self.assertEqual(1, import_mock.call_count)

    @mock.patch('stevedore.driver.DriverManager')
    def test_get_flow_with_admin_repo(self, mock_driver):
        admin_repo = mock.MagicMock()
        executor = taskflow_executor.TaskExecutor(self.context,
                                                  self.task_repo,
                                                  self.image_repo,
                                                  self.image_factory,
                                                  admin_repo=admin_repo)
        self.assertEqual(mock_driver.return_value.driver,
                         executor._get_flow(self.task))
        mock_driver.assert_called_once_with(
            'glance.flows', self.task.type,
            invoke_on_load=True,
            invoke_kwds={'task_id': self.task.task_id,
                         'task_type': self.task.type,
                         'context': self.context,
                         'task_repo': self.task_repo,
                         'image_repo': self.image_repo,
                         'image_factory': self.image_factory,
                         'backend': None,
                         'admin_repo': admin_repo,
                         'uri': 'http://cloud.foo/image.qcow2'})

    @mock.patch('stevedore.driver.DriverManager')
    @mock.patch.object(taskflow_executor, 'LOG')
    def test_get_flow_fails(self, mock_log, mock_driver):
        mock_driver.side_effect = IndexError('fail')
        executor = taskflow_executor.TaskExecutor(self.context,
                                                  self.task_repo,
                                                  self.image_repo,
                                                  self.image_factory)
        self.assertRaises(IndexError, executor._get_flow, self.task)
        mock_log.exception.assert_called_once_with(
            'Task initialization failed: %s', 'fail')
